@@ -7,7 +7,11 @@
 extern "C" {
 #endif
 
+#ifndef __MINGW64__
 #define API __attribute__((visibility("default")))
+#else
+#define API __declspec(dllexport)
+#endif
 #define ALLOC __attribute__((malloc)) __attribute__((warn_unused_result))
 
 // ncdirect_init() will call setlocale() to inspect the current locale. If
@@ -492,8 +496,8 @@ ncdirect_canbraille(const struct ncdirect* nc){
 API bool ncdirect_canget_cursor(const struct ncdirect* nc)
   __attribute__ ((nonnull (1)));
 
-#undef ALLOC
 #undef API
+#undef ALLOC
 
 #ifdef __cplusplus
 }
