@@ -350,24 +350,13 @@ dragonfly_ncneofetch(fetched_info* fi){
 
 static const distro_info*
 xnu_ncneofetch(fetched_info* fi){
-  static distro_info fbsd = {
+  static distro_info xnu = {
     .name = "OS X",
     .logofile = "/System/Library/PrivateFrameworks/LoginUIKit.framework/Versions/A/Frameworks/LoginUICore.framework/Versions/A/Resources/apple@2x.png",
   };
   fi->neologo = get_neofetch_art("Darwin");
-  #ifdef __APPLE__
-  #define PREFIX "macOS "
-  char osver[30] = PREFIX; // shrug
-  size_t oldlenp = sizeof(osver) - strlen(PREFIX);
-  if(sysctlbyname("kern.osproductversion", osver + strlen(PREFIX),
-                  &oldlenp, NULL, 0) == 0){
-    fi->distro_pretty = strdup(osver);
-  }
-  #undef PREFIX
-  #else
-  fi->distro_pretty = strdup("macOS");
-  #endif
-  return &fbsd;
+  fi->distro_pretty = notcurses_osversion();
+  return &xnu;
 }
 
 static int
@@ -607,12 +596,13 @@ display_thread(void* vmarshal){
         pixeling = true;
       }
       struct ncvisual_options vopts = {
+        .n = notcurses_stdplane(m->nc),
         .x = NCALIGN_CENTER,
         .blitter = pixeling ? NCBLIT_PIXEL : NCBLIT_3x2,
         .scaling = pixeling ? NCSCALE_NONE : NCSCALE_SCALE_HIRES,
-        .flags = NCVISUAL_OPTION_HORALIGNED,
+        .flags = NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_CHILDPLANE,
       };
-      struct ncplane* iplane = ncvisual_render(m->nc, ncv, &vopts);
+      struct ncplane* iplane = ncvisual_blit(m->nc, ncv, &vopts);
       ncvisual_destroy(ncv);
       if(iplane){
         int x = ncplane_x(iplane);
