@@ -659,8 +659,8 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
   }
   int ymax = vopts->leny / bset->height;
   int xmax = vopts->lenx / bset->width;
-  int dimy = vopts->leny > 0 ? ymax : ncdirect_dim_y(n);
-  int dimx = vopts->lenx > 0 ? xmax : ncdirect_dim_x(n);
+  unsigned dimy = vopts->leny > 0 ? ymax : ncdirect_dim_y(n);
+  unsigned dimx = vopts->lenx > 0 ? xmax : ncdirect_dim_x(n);
   int disprows, dispcols, outy;
   if(vopts->scaling != NCSCALE_NONE && vopts->scaling != NCSCALE_NONE_HIRES){
     if(bset->geom != NCBLIT_PIXEL){
@@ -720,6 +720,10 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
   if(!ncdv){
     return NULL;
   }
+  if((ncdv->tam = create_tam(ncplane_dim_y(ncdv), ncplane_dim_x(ncdv))) == NULL){
+    free_plane(ncdv);
+    return NULL;
+  }
   blitterargs bargs = {};
   bargs.flags = vopts->flags;
   if(vopts->flags & NCVISUAL_OPTION_ADDALPHA){
@@ -733,7 +737,7 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
     }
     ncdv->sprite = bargs.u.pixel.spx;
   }
-  if(ncvisual_blit(ncv, disprows, dispcols, ncdv, bset, &bargs)){
+  if(ncvisual_blit_internal(ncv, disprows, dispcols, ncdv, bset, &bargs)){
     free_plane(ncdv);
     return NULL;
   }
@@ -853,6 +857,9 @@ ncdirect_stop_minimal(void* vnc){
     ret |= tcsetattr(nc->tcache.ttyfd, TCSANOW, nc->tcache.tpreserved);
   }
   ret |= ncdirect_flush(nc);
+#ifndef __MINGW64__
+  del_curterm(cur_term);
+#endif
   return ret;
 }
 
@@ -902,9 +909,10 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   }else{
     loglevel = NCLOGLEVEL_SILENT;
   }
+  set_loglevel_from_env(&loglevel);
   int cursor_y = -1;
   int cursor_x = -1;
-  if(interrogate_terminfo(&ret->tcache, termtype, ret->ttyfp, utf8, 1,
+  if(interrogate_terminfo(&ret->tcache, ret->ttyfp, utf8, 1,
                           flags & NCDIRECT_OPTION_INHIBIT_CBREAK,
                           0, &cursor_y, &cursor_x, &ret->stats, 0, 0, 0, 0,
                           flags & NCDIRECT_OPTION_DRAIN_INPUT)){
@@ -991,6 +999,9 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
   uint32_t id;
   int oldx = xstart;
   while((id = ncdirect_getc_blocking(n, &ni)) != (uint32_t)-1){
+    if(ni.evtype == NCTYPE_RELEASE){
+      continue;
+    }
     if(id == NCKEY_EOF || id == NCKEY_ENTER){
       if(id == NCKEY_ENTER){
         if(fputc('\n', n->ttyfp) < 0){
@@ -1022,6 +1033,8 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
       wpos = 1;
     }else if(id == 'e' && ni.ctrl){
       wpos = wused - 1;
+    }else if(nckey_supppuab_p(ni.id)){
+      continue;
     }else{
       if(wspace - 1 < wused){
         wspace += BUFSIZ;
