@@ -103,7 +103,7 @@ utf8_egc_len(const char* gcluster, int* colcount){
     r = mbrtowc(&wc, gcluster, MB_CUR_MAX, &mbt);
     if(r < 0){
       // FIXME probably ought escape this somehow
-      logerror("Invalid UTF8: %s\n", gcluster);
+      logerror("invalid UTF8: %s\n", gcluster);
       return -1;
     }
     if(prevw && !injoin && uc_is_grapheme_break(prevw, wc)){
@@ -114,8 +114,11 @@ utf8_egc_len(const char* gcluster, int* colcount){
       if(iswspace(wc)){ // newline or tab
         return ret + 1;
       }
-      logerror("Prohibited or invalid Unicode: 0x%x\n", wc);
-      return -1;
+      if(iswcntrl(wc)){
+        logerror("prohibited or invalid Unicode: 0x%x\n", wc);
+        return -1;
+      }
+      cols = 1;
     }
     injoin = (wc == L'\u200d');
     *colcount += cols;
@@ -304,15 +307,17 @@ egcpool_extended_gcluster(const egcpool* pool, const nccell* c) {
 // contents in 'dst'.
 static inline int
 egcpool_dup(egcpool* dst, const egcpool* src){
-  char* tmp;
-  if((tmp = (char*)realloc(dst->pool, src->poolsize)) == NULL){
-    return -1;
+  if(src->pool){
+    char* tmp;
+    if((tmp = (char*)realloc(dst->pool, src->poolsize)) == NULL){
+      return -1;
+    }
+    dst->pool = tmp;
+    memcpy(dst->pool, src->pool, src->poolsize);
   }
-  dst->pool = tmp;
   dst->poolsize = src->poolsize;
   dst->poolused = src->poolused;
   dst->poolwrite = src->poolwrite;
-  memcpy(dst->pool, src->pool, src->poolsize);
   return 0;
 }
 
