@@ -8,6 +8,8 @@
 #endif
 #ifdef __MINGW64__
 #include <winsock2.h>
+#else
+#include <spawn.h>
 #endif
 #if (defined(__linux__))
 #include <linux/wait.h>
@@ -89,7 +91,7 @@ ncfdplane_create_internal(ncplane* n, const ncfdplane_options* opts, int fd,
                           ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn,
                           bool thread){
   if(opts->flags > 0){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
+    logwarn("Provided unsupported flags %016" PRIx64 "\n", opts->flags);
   }
   ncfdplane* ret = malloc(sizeof(*ret));
   if(ret == NULL){
@@ -142,54 +144,94 @@ int ncfdplane_destroy(ncfdplane* n){
   return ret;
 }
 
+#ifndef __MINGW64__
+// get 2 pipes, and ensure they're both set to close-on-exec
+static int
+lay_pipes(int pipes[static 2]){
+#ifdef __linux__
+  if(pipe2(pipes, O_CLOEXEC)){ // can't use O_NBLOCK here (affects client)
+#else
+  if(pipe(pipes)){
+#endif
+    return -1;
+  }
+#ifndef __linux__
+  if(set_fd_cloexec(pipes[0], 1, NULL) || set_fd_cloexec(pipes[1], 1, NULL)){
+    close(pipes[0]);
+    close(pipes[1]);
+    return -1;
+  }
+#endif
+  return 0;
+}
+
 // ncsubproc creates a pipe, retaining the read end. it clone()s a subprocess,
 // getting a pidfd. the subprocess dup2()s the write end of the pipe onto file
 // descriptors 1 and 2, exec()s, and begins running. the parent creates an
 // ncfdplane around the read end, involving creation of a new thread. the
 // parent then returns.
 static pid_t
-launch_pipe_process(int* pipefd, int* pidfd){
-#ifndef __MINGW64__
+launch_pipe_process(int* pipefd, int* pidfd, unsigned usepath,
+                    const char* bin,  char* const arg[], char* const env[]){
   *pidfd = -1;
   int pipes[2];
-#if (defined(__linux__))
-  if(pipe2(pipes, O_CLOEXEC)){ // can't use O_NBLOCK here (affects client)
-#else
-  if(pipe(pipes)){ // FIXME manually set O_CLOEXEC
-#endif
+  if(lay_pipes(pipes)){
     return -1;
   }
   pid_t p = -1;
-  // on linux, we try to use the brand-new pidfd capability via clone3(). if
-  // that fails, fall through to fork(), which is all we try to use on freebsd.
-  // FIXME clone3 is not yet supported on debian sparc64/alpha as of 2020-07
 #ifdef USING_PIDFD
+  // on linux, we try to use the brand-new pidfd capability via clone3(). if
+  // that fails, fall through to posix_spawn(), our only option on freebsd.
+  // FIXME clone3 is not yet supported on debian sparc64/alpha as of 2020-07
   struct clone_args clargs;
   memset(&clargs, 0, sizeof(clargs));
   clargs.pidfd = (uintptr_t)pidfd;
   clargs.flags = CLONE_CLEAR_SIGHAND | CLONE_FS | CLONE_PIDFD;
   clargs.exit_signal = SIGCHLD;
   p = syscall(__NR_clone3, &clargs, sizeof(clargs));
-#endif
-  if(p < 0){
-    p = fork();
-  }
   if(p == 0){ // child
     if(dup2(pipes[1], STDOUT_FILENO) < 0 || dup2(pipes[1], STDERR_FILENO) < 0){
       logerror("Couldn't dup() %d (%s)\n", pipes[1], strerror(errno));
       exit(EXIT_FAILURE);
     }
-  }else if(p > 0){ // parent
+    if(env){
+      execvpe(bin, arg, env);
+    }else if(usepath){
+      execvp(bin, arg);
+    }else{
+      execv(bin, arg);
+    }
+    exit(EXIT_FAILURE);
+  }else if(p < 0){
+    logwarn("clone3() failed (%s), using posix_spawn()\n", strerror(errno));
+  }
+#endif
+  if(p < 0){
+    posix_spawn_file_actions_t factions;
+    if(posix_spawn_file_actions_init(&factions)){
+      logerror("couldn't initialize spawn file actions\n");
+      return -1;
+    }
+    posix_spawn_file_actions_adddup2(&factions, pipes[1], STDOUT_FILENO);
+    posix_spawn_file_actions_adddup2(&factions, pipes[1], STDERR_FILENO);
+    int r;
+    if(usepath){
+      r = posix_spawnp(&p, bin, &factions, NULL, arg, env);
+    }else{
+      r = posix_spawn(&p, bin, &factions, NULL, arg, env);
+    }
+    if(r){
+      logerror("posix_spawn %s failed (%s)\n", bin, strerror(errno));
+    }
+    posix_spawn_file_actions_destroy(&factions);
+  }
+  if(p > 0){ // parent
     *pipefd = pipes[0];
     set_fd_nonblocking(*pipefd, 1, NULL);
   }
   return p;
-#else
-  (void)pipefd;
-  (void)pidfd;
-  return -1;
-#endif
 }
+#endif
 
 // nuke the just-spawned process, and reap it. called before the subprocess
 // reader thread is launched (which otherwise reaps the subprocess).
@@ -294,9 +336,11 @@ ncsubproc_launch(ncplane* n, ncsubproc* ret, const ncsubproc_options* opts, int
   return ret->nfp;
 }
 
-ncsubproc* ncsubproc_createv(ncplane* n, const ncsubproc_options* opts,
-                             const char* bin,  char* const arg[],
-                             ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn){
+// use of env implies usepath
+static ncsubproc*
+ncexecvpe(ncplane* n, const ncsubproc_options* opts, unsigned usepath,
+          const char* bin,  char* const arg[], char* const env[],
+          ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn){
   ncsubproc_options zeroed = {};
   if(!opts){
     opts = &zeroed;
@@ -305,19 +349,17 @@ ncsubproc* ncsubproc_createv(ncplane* n, const ncsubproc_options* opts,
     return NULL;
   }
   if(opts->flags > 0){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
+    logwarn("Provided unsupported flags %016" PRIx64 "\n", opts->flags);
   }
+#ifndef __MINGW64__
   int fd = -1;
   ncsubproc* ret = malloc(sizeof(*ret));
   if(ret == NULL){
     return NULL;
   }
   memset(ret, 0, sizeof(*ret));
-  ret->pid = launch_pipe_process(&fd, &ret->pidfd);
-  if(ret->pid == 0){
-    execv(bin, arg);
-    exit(EXIT_FAILURE);
-  }else if(ret->pid < 0){
+  ret->pid = launch_pipe_process(&fd, &ret->pidfd, usepath, bin, arg, env);
+  if(ret->pid < 0){
     free(ret);
     return NULL;
   }
@@ -327,89 +369,35 @@ ncsubproc* ncsubproc_createv(ncplane* n, const ncsubproc_options* opts,
     return NULL;
   }
   return ret;
+#else
+  // FIXME use CreateProcess()
+  (void)n;
+  (void)usepath;
+  (void)bin;
+  (void)arg;
+  (void)env;
+  (void)cbfxn;
+  (void)donecbfxn;
+  return NULL;
+#endif
+}
+
+ncsubproc* ncsubproc_createv(ncplane* n, const ncsubproc_options* opts,
+                             const char* bin,  char* const arg[],
+                             ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn){
+  return ncexecvpe(n, opts, 0, bin, arg, NULL, cbfxn, donecbfxn);
 }
 
 ncsubproc* ncsubproc_createvp(ncplane* n, const ncsubproc_options* opts,
                               const char* bin,  char* const arg[],
                               ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn){
-  ncsubproc_options zeroed = {};
-  if(!opts){
-    opts = &zeroed;
-  }
-  if(!cbfxn || !donecbfxn){
-    return NULL;
-  }
-  if(opts->flags > 0){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
-  }
-  int fd = -1;
-  ncsubproc* ret = malloc(sizeof(*ret));
-  if(ret == NULL){
-    return NULL;
-  }
-  memset(ret, 0, sizeof(*ret));
-  ret->pid = launch_pipe_process(&fd, &ret->pidfd);
-  if(ret->pid == 0){
-    execvp(bin, arg);
-    exit(EXIT_FAILURE);
-  }else if(ret->pid < 0){
-    free(ret);
-    return NULL;
-  }
-  if((ret->nfp = ncsubproc_launch(n, ret, opts, fd, cbfxn, donecbfxn)) == NULL){
-    kill_and_wait_subproc(ret->pid, ret->pidfd, NULL);
-    free(ret);
-    return NULL;
-  }
-  return ret;
+  return ncexecvpe(n, opts, 1, bin, arg, NULL, cbfxn, donecbfxn);
 }
 
 ncsubproc* ncsubproc_createvpe(ncplane* n, const ncsubproc_options* opts,
                        const char* bin,  char* const arg[], char* const env[],
                        ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn){
-  ncsubproc_options zeroed = {};
-  if(!opts){
-    opts = &zeroed;
-  }
-  if(!cbfxn || !donecbfxn){
-    return NULL;
-  }
-  if(opts->flags > 0){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
-  }
-  int fd = -1;
-  ncsubproc* ret = malloc(sizeof(*ret));
-  if(ret == NULL){
-    return NULL;
-  }
-  memset(ret, 0, sizeof(*ret));
-  ret->pid = launch_pipe_process(&fd, &ret->pidfd);
-  if(ret->pid == 0){
-#ifdef __linux__
-    execvpe(bin, arg, env);
-#elif defined(__APPLE__) || defined(__gnu_hurd__)
-    (void)env;
-    execvp(bin, arg); // FIXME env?
-#elif defined(__MINGW64__)
-    (void)arg;
-    (void)env;
-#else
-    exect(bin, arg, env);
-#endif
-    logerror("Error execing %s (%s?)\n", bin, strerror(errno));
-    exit(EXIT_FAILURE);
-  }else if(ret->pid < 0){
-    logerror("Error launching process (%s?)\n", strerror(errno));
-    free(ret);
-    return NULL;
-  }
-  pthread_mutex_init(&ret->lock, NULL);
-  if((ret->nfp = ncsubproc_launch(n, ret, opts, fd, cbfxn, donecbfxn)) == NULL){
-    kill_and_wait_subproc(ret->pid, ret->pidfd, NULL);
-    free(ret);
-    return NULL;
-  }
-  return ret;
+  return ncexecvpe(n, opts, 1, bin, arg, env, cbfxn, donecbfxn);
 }
 
 int ncsubproc_destroy(ncsubproc* n){
