@@ -23,7 +23,6 @@
 // redirected from a file (NCOPTION_TOSS_INPUT)
 
 // FIXME still need to:
-//  integrate main specials trie with automaton, enable input_errors
 //  probably want pipes/eventfds rather than SIGCONT
 
 static sig_atomic_t resize_seen;
@@ -50,6 +49,12 @@ typedef struct cursorloc {
   int y, x;             // 0-indexed cursor location
 } cursorloc;
 
+#ifndef __MINGW64__
+typedef int ipipe;
+#else
+typedef HANDLE ipipe;
+#endif
+
 // local state for the input thread. don't put this large struct on the stack.
 typedef struct inputctx {
   int stdinfd;          // bulk in fd. always >= 0 (almost always 0). we do not
@@ -100,9 +105,10 @@ typedef struct inputctx {
   unsigned drain;     // drain away bulk input?
   ncsharedstats *stats; // stats shared with notcurses context
 
-  int readypipes[2];  // pipes[0]: poll()able fd indicating the presence of user input
+  ipipe readypipes[2];  // pipes[0]: poll()able fd indicating the presence of user input
   struct initial_responses* initdata;
   struct initial_responses* initdata_complete;
+  bool failed;          // error initializing input automaton, abort
 } inputctx;
 
 static inline void
@@ -354,6 +360,36 @@ prep_special_keys(inputctx* ictx){
   return 0;
 }
 
+// starting from the current amata match point, match any necessary prefix, then
+// extract the (possibly empty) content, then match the follow. as we are only
+// called from a callback context, and know we've been properly matched, there
+// is no error-checking per se (we do require prefix/follow matches, but if
+// missed, we just return NULL). indicate empty prefix with "", not NULL.
+// updates ictx->amata.matchstart to be pointing past the follow. follow ought
+// not be NUL.
+static char*
+amata_next_kleene(automaton* amata, const char* prefix, char follow){
+  char c;
+  while( (c = *prefix++) ){
+    if(*amata->matchstart != c){
+      logerror("matchstart didn't match prefix (%c vs %c)\n", c, *amata->matchstart);
+      return NULL;
+    }
+    ++amata->matchstart;
+  }
+  // prefix has been matched. mark start of string and find follow.
+  const unsigned char* start = amata->matchstart;
+  while(*amata->matchstart != follow){
+    ++amata->matchstart;
+  }
+  char* ret = malloc(amata->matchstart - start + 1);
+  if(ret){
+    memcpy(ret, start, amata->matchstart - start);
+    ret[amata->matchstart - start] = '\0';
+  }
+  return ret;
+}
+
 // starting from the current amata match point, match any necessary prefix, then
 // extract the numeric (possibly empty), then match the follow. as we are only
 // called from a callback context, and know we've been properly matched, there
@@ -395,25 +431,7 @@ amata_next_numeric(automaton* amata, const char* prefix, char follow){
 // either a match failure or an alloc failure.
 static char*
 amata_next_string(automaton* amata, const char* prefix){
-  char c;
-  while( (c = *prefix++) ){
-    if(*amata->matchstart != c){
-      logerror("matchstart didn't match prefix (%c vs %c)\n", c, *amata->matchstart);
-      return NULL;
-    }
-    ++amata->matchstart;
-  }
-  // prefix has been matched. mark start of string and find follow.
-  const unsigned char* start = amata->matchstart;
-  while(*amata->matchstart != '\x1b'){
-    ++amata->matchstart;
-  }
-  char* ret = malloc(amata->matchstart - start + 1);
-  if(ret){
-    memcpy(ret, start, amata->matchstart - start);
-    ret[amata->matchstart - start] = '\0';
-  }
-  return ret;
+  return amata_next_kleene(amata, prefix, '\x1b');
 }
 
 static inline void
@@ -426,10 +444,16 @@ send_synth_signal(int sig){
 }
 
 static void
-mark_pipe_ready(int pipes[static 2]){
+mark_pipe_ready(ipipe pipes[static 2]){
   char sig = 1;
+#ifndef __MINGW64__
   if(write(pipes[1], &sig, sizeof(sig)) != 1){
     logwarn("error writing to readypipe (%d) (%s)\n", pipes[1], strerror(errno));
+#else
+  DWORD wrote;
+  if(!WriteFile(pipes[1], &sig, sizeof(sig), &wrote, NULL) || wrote != sizeof(sig)){
+    logwarn("error writing to readypipe\n");
+#endif
   }
 }
 
@@ -437,6 +461,7 @@ static void
 load_ncinput(inputctx* ictx, const ncinput *tni, int synthsig){
   inc_input_events(ictx);
   if(ictx->drain){
+    send_synth_signal(synthsig);
     return;
   }
   pthread_mutex_lock(&ictx->ilock);
@@ -597,6 +622,8 @@ kitty_functional(uint32_t val){
       val = NCKEY_MEDIA_PLAY + val - 57428;
     }else if(val >= 57399 && val <= 57408){
       val = '0' + val - 57399;
+    }else if(val >= 57441 && val <= 57452){
+      val = NCKEY_LSHIFT + val - 57441;
     }else switch(val){
       case 57358: val = NCKEY_CAPS_LOCK; break;
       case 57400: val = '1'; break;
@@ -624,6 +651,10 @@ kitty_functional(uint32_t val){
       case 57426: val = NCKEY_DEL; break;
       case 57427: val = NCKEY_BEGIN; break;
     }
+  }else{
+    switch(val){
+      case 0xd: val = NCKEY_ENTER; break;
+    }
   }
   return val;
 }
@@ -689,6 +720,31 @@ kitty_cb(inputctx* ictx){
   return 2;
 }
 
+static uint32_t
+legacy_functional(uint32_t id){
+  switch(id){
+    case 2: id = NCKEY_INS; break;
+    case 3: id = NCKEY_DEL; break;
+    case 5: id = NCKEY_PGUP; break;
+    case 6: id = NCKEY_PGDOWN; break;
+    case 7: id = NCKEY_HOME; break;
+    case 8: id = NCKEY_END; break;
+    case 11: id = NCKEY_F01; break;
+    case 12: id = NCKEY_F02; break;
+    case 13: id = NCKEY_F03; break;
+    case 14: id = NCKEY_F04; break;
+    case 15: id = NCKEY_F05; break;
+    case 17: id = NCKEY_F06; break;
+    case 18: id = NCKEY_F07; break;
+    case 19: id = NCKEY_F08; break;
+    case 20: id = NCKEY_F09; break;
+    case 21: id = NCKEY_F10; break;
+    case 23: id = NCKEY_F11; break;
+    case 24: id = NCKEY_F12; break;
+  }
+  return id;
+}
+
 static int
 kitty_cb_functional(inputctx* ictx){
   unsigned val = amata_next_numeric(&ictx->amata, "\x1b[", ';');
@@ -696,31 +752,21 @@ kitty_cb_functional(inputctx* ictx){
   unsigned ev = amata_next_numeric(&ictx->amata, "", '~');
   uint32_t kval = kitty_functional(val);
   if(kval == val){
-    switch(val){
-      case 2: kval = NCKEY_INS; break;
-      case 3: kval = NCKEY_DEL; break;
-      case 5: kval = NCKEY_PGUP; break;
-      case 6: kval = NCKEY_PGDOWN; break;
-      case 7: kval = NCKEY_HOME; break;
-      case 8: kval = NCKEY_END; break;
-      case 11: kval = NCKEY_F01; break;
-      case 12: kval = NCKEY_F02; break;
-      case 13: kval = NCKEY_F03; break;
-      case 14: kval = NCKEY_F04; break;
-      case 15: kval = NCKEY_F05; break;
-      case 17: kval = NCKEY_F06; break;
-      case 18: kval = NCKEY_F07; break;
-      case 19: kval = NCKEY_F08; break;
-      case 20: kval = NCKEY_F09; break;
-      case 21: kval = NCKEY_F10; break;
-      case 23: kval = NCKEY_F11; break;
-      case 24: kval = NCKEY_F12; break;
-    }
+    kval = legacy_functional(val);
   }
   kitty_kbd(ictx, kval, mods, ev);
   return 2;
 }
 
+static int
+wezterm_cb(inputctx* ictx){
+  unsigned val = amata_next_numeric(&ictx->amata, "\x1b[", ';');
+  unsigned mods = amata_next_numeric(&ictx->amata, "", '~');
+  uint32_t kval = legacy_functional(val);
+  kitty_kbd(ictx, kval, mods, 0);
+  return 2;
+}
+
 static int
 kitty_cb_f1(inputctx* ictx){
   unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", ':');
@@ -871,21 +917,136 @@ handoff_initial_responses(inputctx* ictx){
   loginfo("handing off initial responses\n");
 }
 
+// if XTSMGRAPHICS responses were provided, but DA1 didn't advertise sixel
+// support, we need scrub those responses, lest we try to use Sixel.
+static inline void
+scrub_sixel_responses(struct initial_responses* idata){
+  if(idata->color_registers || idata->sixelx || idata->sixely){
+    logwarn("answered XTSMGRAPHICS, but no sixel in DA1\n");
+    idata->color_registers = 0;
+    idata->sixelx = 0;
+    idata->sixely = 0;
+  }
+}
+
+// annoyingly, alacritty (well, branches of alacritty) supports Sixel, but
+// does not indicate this in their Primary Device Attributes response (there
+// is no room for attributes in a VT102-style DA1, which alacritty uses).
+// so, iff we've determined we're alacritty, don't scrub out Sixel details.
 static int
-da1_cb(inputctx* ictx){
+da1_vt102_cb(inputctx* ictx){
   loginfo("read primary device attributes\n");
   if(ictx->initdata){
+    if(ictx->initdata->qterm != TERMINAL_ALACRITTY){
+      scrub_sixel_responses(ictx->initdata);
+    }
     handoff_initial_responses(ictx);
   }
   return 1;
 }
 
+static int
+da1_cb(inputctx* ictx){
+  loginfo("read primary device attributes\n");
+  if(ictx->initdata){
+    scrub_sixel_responses(ictx->initdata);
+    handoff_initial_responses(ictx);
+  }
+  return 1;
+}
+
+static int
+da1_attrs_cb(inputctx* ictx){
+  loginfo("read primary device attributes\n");
+  unsigned val = amata_next_numeric(&ictx->amata, "\x1b[?", ';');
+  char* attrlist = amata_next_kleene(&ictx->amata, "", 'c');
+  logdebug("DA1: %u [%s]\n", val, attrlist);
+  if(ictx->initdata){
+    int foundsixel = 0;
+    unsigned curattr = 0;
+    for(const char* a = attrlist ; *a ; ++a){
+      if(isdigit(*a)){
+        curattr *= 10;
+        curattr += *a - '0';
+      }else if(*a == ';'){
+        if(curattr == 4){
+          foundsixel = 1;
+          if(ictx->initdata->color_registers <= 0){
+            ictx->initdata->color_registers = 256;
+          }
+        }
+        curattr = 0;
+      }
+    }
+    if(curattr == 4){
+      foundsixel = 1;
+      if(ictx->initdata->color_registers <= 0){
+        ictx->initdata->color_registers = 256;
+      }
+    }
+    if(!foundsixel){
+      scrub_sixel_responses(ictx->initdata);
+    }
+    handoff_initial_responses(ictx);
+  }
+  free(attrlist);
+  return 1;
+}
+
+// GNU screen primarily identifies itself via an "83" as the first parameter
+// of its DA2 reply. the version is the second parameter.
+static int
+da2_screen_cb(inputctx* ictx){
+  if(ictx->initdata == NULL){
+    return 2;
+  }
+  if(ictx->initdata->qterm != TERMINAL_UNKNOWN){
+    logwarn("already identified term (%d)\n", ictx->initdata->qterm);
+    return 2;
+  }
+  unsigned ver = amata_next_numeric(&ictx->amata, "\x1b[>83;", ';');
+  if(ver < 10000){
+    logwarn("version %u doesn't look like GNU screen\n", ver);
+    return 2;
+  }
+  char verstr[9]; // three two-digit components plus two delims
+  int s = snprintf(verstr, sizeof(verstr), "%u.%02u.%02u",
+                   ver / 10000, ver / 100 % 100, ver % 100);
+  if(s < 0 || (unsigned)s >= sizeof(verstr)){
+    logwarn("bad screen version %u\n", ver);
+    return 2;
+  }
+  ictx->initdata->version = strdup(verstr);
+  ictx->initdata->qterm = TERMINAL_GNUSCREEN;
+  return 2;
+}
+
+// we use secondary device attributes to recognize the alacritty crate
+// version, and to ascertain the version of old, pre-XTVERSION XTerm.
 static int
 da2_cb(inputctx* ictx){
   loginfo("read secondary device attributes\n");
   if(ictx->initdata == NULL){
     return 2;
   }
+  amata_next_numeric(&ictx->amata, "\x1b[>", ';');
+  unsigned pv = amata_next_numeric(&ictx->amata, "", ';');
+  int maj, min, patch;
+  if(pv == 0){
+    return 2;
+  }
+  if(ictx->initdata->qterm == TERMINAL_XTERM){
+    if(ictx->initdata->version == NULL){
+      char ver[8];
+      int s = snprintf(ver, sizeof(ver), "%u", pv);
+      if(s < 0 || (unsigned)s >= sizeof(ver)){
+        logerror("bad version: %u\n", pv);
+      }else{
+        ictx->initdata->version = strdup(ver);
+      }
+      return 2;
+    }
+  }
   // SDA yields up Alacritty's crate version, but it doesn't unambiguously
   // identify Alacritty. If we've got any other version information, don't
   // use this. use the second parameter (Pv).
@@ -901,12 +1062,6 @@ da2_cb(inputctx* ictx){
             termname ? termname : "unset");
     return 2;
   }
-  amata_next_numeric(&ictx->amata, "\x1b[>", ';');
-  unsigned pv = amata_next_numeric(&ictx->amata, "", ';');
-  int maj, min, patch;
-  if(pv == 0){
-    return 2;
-  }
   maj = pv / 10000;
   min = (pv % 10000) / 100;
   patch = pv % 100;
@@ -1066,6 +1221,8 @@ tcap_cb(inputctx* ictx){
       ictx->initdata->qterm = TERMINAL_MLTERM;
     }else if(strcasecmp(tn, "787465726d2d6b69747479") == 0){
       ictx->initdata->qterm = TERMINAL_KITTY;
+    }else if(strcasecmp(tn, "787465726D2D323536636F6C6F72") == 0){
+      ictx->initdata->qterm = TERMINAL_XTERM; // "xterm-256color"
     }else{
       logdebug("unknown terminal name %s\n", tn);
     }
@@ -1109,10 +1266,10 @@ build_cflow_automaton(inputctx* ictx){
     // CSI (\e[)
     { "[<\\N;\\N;\\NM", mouse_press_cb, },
     { "[<\\N;\\N;\\Nm", mouse_release_cb, },
-    { "[\\N;\\NR", cursor_location_cb, },
     // technically these must begin with "4" or "8"; enforce in callbacks
     { "[\\N;\\N;\\Nt", geom_cb, },
     { "[\\Nu", kitty_cb_simple, },
+    { "[\\N;\\N~", wezterm_cb, },
     { "[\\N;\\Nu", kitty_cb, },
     { "[\\N;\\N:\\Nu", kitty_cb_complex, },
     { "[\\N;\\N;\\N~", xtmodkey_cb, },
@@ -1129,26 +1286,29 @@ build_cflow_automaton(inputctx* ictx){
     { "[1;\\N:\\NF", kitty_cb_end, },
     { "[1;\\N:\\NH", kitty_cb_home, },
     { "[?\\Nu", kitty_keyboard_cb, },
-    { "[?1;2c", da1_cb, }, // CSI ? 1 ; 2 c ("VT100 with Advanced Video Option")
-    { "[?1;0c", da1_cb, }, // CSI ? 1 ; 0 c ("VT101 with No Options")
-    { "[?4;6c", da1_cb, }, // CSI ? 4 ; 6 c ("VT132 with Advanced Video and Graphics")
-    { "[?6c", da1_cb, },   // CSI ? 6 c ("VT102")
-    { "[?7c", da1_cb, },   // CSI ? 7 c ("VT131")
-    { "[?12;\\Dc", da1_cb, }, // CSI ? 1 2 ; Ps c ("VT125")
-    { "[?60;\\Dc", da1_cb, }, // CSI ? 6 0 ; Ps c (kmscon)
-    { "[?62;\\Dc", da1_cb, }, // CSI ? 6 2 ; Ps c ("VT220")
-    { "[?63;\\Dc", da1_cb, }, // CSI ? 6 3 ; Ps c ("VT320")
-    { "[?64;\\Dc", da1_cb, }, // CSI ? 6 4 ; Ps c ("VT420")
-    { "[?65;\\Dc", da1_cb, }, // CSI ? 6 5 ; Ps c (WezTerm, VT5xx?)
+    { "[?2026;\\N$y", decrpm_asu_cb, },
+    { "[\\N;\\NR", cursor_location_cb, },
     { "[?1;1S", NULL, }, // negative cregs XTSMGRAPHICS
     { "[?1;2S", NULL, }, // negative cregs XTSMGRAPHICS
     { "[?1;3;0S", NULL, }, // negative cregs XTSMGRAPHICS
-    { "[?2026;\\N$y", decrpm_asu_cb, },
     { "[?2;1S", NULL, }, // negative pixels XTSMGRAPHICS
     { "[?2;2S", NULL, }, // negative pixels XTSMGRAPHICS
     { "[?2;3;0S", NULL, }, // negative pixels XTSMGRAPHICS
+    { "[?6c", da1_vt102_cb, },   // CSI ? 6 c ("VT102")
+    { "[?7c", da1_cb, },   // CSI ? 7 c ("VT131")
+    { "[?1;0c", da1_cb, }, // CSI ? 1 ; 0 c ("VT101 with No Options")
+    { "[?1;2c", da1_cb, }, // CSI ? 1 ; 2 c ("VT100 with Advanced Video Option")
+    { "[?4;6c", da1_cb, }, // CSI ? 4 ; 6 c ("VT132 with Advanced Video and Graphics")
+    // CSI ? 1 2 ; Ps c ("VT125")
+    // CSI ? 6 0 ; Ps c (kmscon)
+    // CSI ? 6 2 ; Ps c ("VT220")
+    // CSI ? 6 3 ; Ps c ("VT320")
+    // CSI ? 6 4 ; Ps c ("VT420")
+    // CSI ? 6 5 ; Ps c (WezTerm, VT5xx?)
+    { "[?\\N;\\Dc", da1_attrs_cb, },
     { "[?1;0;\\NS", xtsmgraphics_cregs_cb, },
     { "[?2;0;\\N;\\NS", xtsmgraphics_sixel_cb, },
+    { "[>83;\\N;0c", da2_screen_cb, },
     { "[>\\N;\\N;\\Nc", da2_cb, },
     { "[=\\Sc", da3_cb, }, // CSI da3 form as issued by WezTerm
     // DCS (\eP...ST)
@@ -1173,18 +1333,27 @@ build_cflow_automaton(inputctx* ictx){
 }
 
 static void
-endpipes(int pipes[static 2]){
-  if(pipes[0] >= 0){
-    close(pipes[0]);
+closepipe(ipipe p){
+#ifndef __MINGW64__
+  if(p >= 0){
+    close(p);
   }
-  if(pipes[1] >= 0){
-    close(pipes[1]);
+#else
+  if(p){
+    CloseHandle(p);
   }
+#endif
+}
+
+static void
+endpipes(ipipe pipes[static 2]){
+  closepipe(pipes[0]);
+  closepipe(pipes[1]);
 }
 
 // only linux and freebsd13+ have eventfd(), so we'll fall back to pipes sigh.
 static int
-getpipes(int pipes[static 2]){
+getpipes(ipipe pipes[static 2]){
 #ifndef __MINGW64__
 #ifndef __APPLE__
   if(pipe2(pipes, O_CLOEXEC | O_NONBLOCK)){
@@ -1197,19 +1366,21 @@ getpipes(int pipes[static 2]){
     return -1;
   }
   if(set_fd_cloexec(pipes[0], 1, NULL) || set_fd_nonblocking(pipes[0], 1, NULL)){
-    logerror("couldn't prep pipe[0] (%d) (%s)\n", pipes[0], strerror(errno));
+    logerror("couldn't prep pipe[0] (%s)\n", strerror(errno));
     endpipes(pipes);
     return -1;
   }
   if(set_fd_cloexec(pipes[1], 1, NULL) || set_fd_nonblocking(pipes[1], 1, NULL)){
-    logerror("couldn't prep pipe[1] (%d) (%s)\n", pipes[1], strerror(errno));
+    logerror("couldn't prep pipe[1] (%s)\n", strerror(errno));
     endpipes(pipes);
     return -1;
   }
-  // FIXME what to do on windows?
 #endif
 #else // windows
-  pipes[0] = pipes[1] = -1;
+  if(!CreatePipe(&pipes[0], &pipes[1], NULL, BUFSIZ)){
+    logerror("couldn't get pipes\n");
+    return -1;
+  }
 #endif
   return 0;
 }
@@ -1236,6 +1407,7 @@ create_inputctx(tinfo* ti, FILE* infp, int lmargin, int tmargin, int rmargin,
                         if(set_fd_nonblocking(i->stdinfd, 1, &ti->stdio_blocking_save) == 0){
                           i->termfd = tty_check(i->stdinfd) ? -1 : get_tty_fd(infp);
                           memset(i->initdata, 0, sizeof(*i->initdata));
+                          i->initdata->qterm = ti->qterm;
                           i->iread = i->iwrite = i->ivalid = 0;
                           i->cread = i->cwrite = i->cvalid = 0;
                           i->initdata_complete = NULL;
@@ -1254,6 +1426,7 @@ create_inputctx(tinfo* ti, FILE* infp, int lmargin, int tmargin, int rmargin,
                           i->rmargin = rmargin;
                           i->bmargin = bmargin;
                           i->drain = drain;
+                          i->failed = false;
                           logdebug("input descriptors: %d/%d\n", i->stdinfd, i->termfd);
                           return i;
                         }
@@ -1829,11 +2002,9 @@ read_inputs_nblock(inputctx* ictx){
 static void*
 input_thread(void* vmarshall){
   inputctx* ictx = vmarshall;
-  if(prep_all_keys(ictx)){
-    abort(); // FIXME?
-  }
-  if(build_cflow_automaton(ictx)){
-    abort(); // FIXME?
+  if(prep_all_keys(ictx) || build_cflow_automaton(ictx)){
+    ictx->failed = true;
+    handoff_initial_responses(ictx);
   }
   for(;;){
     read_inputs_nblock(ictx);
@@ -1878,7 +2049,13 @@ int stop_inputlayer(tinfo* ti){
 }
 
 int inputready_fd(const inputctx* ictx){
+#ifndef __MINGW64__
   return ictx->readypipes[0];
+#else
+  (void)ictx;
+  logerror("readiness descriptor unavailable on windows\n");
+  return -1;
+#endif
 }
 
 static inline uint32_t
@@ -1902,7 +2079,7 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
     if(ts == NULL){
       pthread_cond_wait(&ictx->icond, &ictx->ilock);
     }else{
-      int r = pthread_cond_timedwait(&ictx->icond, &ictx->ilock, ts);
+      int r = pthread_cond_clockwait(&ictx->icond, &ictx->ilock, CLOCK_MONOTONIC, ts);
       if(r == ETIMEDOUT){
         pthread_mutex_unlock(&ictx->ilock);
         return 0;
@@ -1925,9 +2102,15 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
   }else if(ictx->ivalid){
     logtrace("draining event readiness pipe\n");
     char c;
+#ifndef __MINGW64__
     while(read(ictx->readypipes[0], &c, sizeof(c)) == 1){
-      // FIXME accelerate;
+      // FIXME accelerate?
     }
+#else
+    while(ReadFile(ictx->readypipes[0], &c, sizeof(c), NULL, NULL)){
+      // FIXME accelerate?
+    }
+#endif
   }
   pthread_mutex_unlock(&ictx->ilock);
   if(sendsignal){
@@ -1956,7 +2139,13 @@ delaybound_to_deadline(const struct timespec* ts, struct timespec* absdl){
 uint32_t notcurses_get(notcurses* nc, const struct timespec* ts, ncinput* ni){
   struct timespec absdl;
   delaybound_to_deadline(ts, &absdl);
-  return internal_get(nc->tcache.ictx, ts ? &absdl : NULL, ni);
+  uint32_t id = internal_get(nc->tcache.ictx, ts ? &absdl : NULL, ni);
+  if(ni){
+    if(id == (uint32_t)-1){
+      ni->id = id;
+    }
+  }
+  return id;
 }
 
 // FIXME better performance if we move this within the locked area
@@ -2110,5 +2299,10 @@ struct initial_responses* inputlayer_get_responses(inputctx* ictx){
   iresp = ictx->initdata_complete;
   ictx->initdata_complete = NULL;
   pthread_mutex_unlock(&ictx->ilock);
+  if(ictx->failed){
+    logpanic("aborting after automaton construction failure\n");
+    free(iresp);
+    return NULL;
+  }
   return iresp;
 }
