@@ -8,9 +8,17 @@ extern "C" {
 #include "version.h"
 #include "builddef.h"
 #include "compat/compat.h"
+#include "notcurses/ncport.h"
 #include "notcurses/notcurses.h"
 #include "notcurses/direct.h"
 
+#ifndef __MINGW64__
+#define API __attribute__((visibility("default")))
+#else
+#define API __declspec(dllexport)
+#endif
+#define ALLOC __attribute__((malloc)) __attribute__((warn_unused_result))
+
 // KEY_EVENT is defined by both ncurses.h and wincon.h. since we don't use
 // either definition, kill it before inclusion of ncurses.h.
 #undef KEY_EVENT
@@ -39,9 +47,6 @@ extern "C" {
 #include "lib/fbuf.h"
 #include "lib/gpm.h"
 
-#define API __attribute__((visibility("default")))
-#define ALLOC __attribute__((malloc)) __attribute__((warn_unused_result))
-
 struct sixelmap;
 struct ncvisual_details;
 
@@ -177,28 +182,6 @@ typedef struct ncreel {
   ncreel_options ropts; // copied in ncreel_create()
 } ncreel;
 
-// ncmenu_item and ncmenu_section have internal and (minimal) external forms
-typedef struct ncmenu_int_item {
-  char* desc;           // utf-8 menu item, NULL for horizontal separator
-  ncinput shortcut;     // shortcut, all should be distinct
-  int shortcut_offset;  // column offset with desc of shortcut EGC
-  char* shortdesc;      // description of shortcut, can be NULL
-  int shortdesccols;    // columns occupied by shortcut description
-  bool disabled;        // disabled?
-} ncmenu_int_item;
-
-typedef struct ncmenu_int_section {
-  char* name;             // utf-8 c string
-  int itemcount;
-  ncmenu_int_item* items; // items, NULL iff itemcount == 0
-  ncinput shortcut;       // shortcut, will be underlined if present in name
-  int xoff;               // column offset from beginning of menu bar
-  int bodycols;           // column width of longest item
-  int itemselected;       // current item selected, -1 for no selection
-  int shortcut_offset;    // column offset within name of shortcut EGC
-  int enabled_item_count; // number of enabled items: section is disabled iff 0
-} ncmenu_int_section;
-
 typedef struct ncfdplane {
   ncfdplane_callback cb;      // invoked with fresh hot data
   ncfdplane_done_cb donecb;   // invoked on EOF (if !follow) or error
@@ -230,19 +213,6 @@ typedef struct ncreader {
   bool manage_cursor;         // enable and place a virtual cursor
 } ncreader;
 
-typedef struct ncmenu {
-  ncplane* ncp;
-  int sectioncount;         // must be positive
-  ncmenu_int_section* sections; // NULL iff sectioncount == 0
-  int unrolledsection;      // currently unrolled section, -1 if none
-  int headerwidth;          // minimum space necessary to display all sections
-  uint64_t headerchannels;  // styling for header
-  uint64_t dissectchannels; // styling for disabled section headers
-  uint64_t sectionchannels; // styling for sections
-  uint64_t disablechannels; // styling for disabled entries
-  bool bottom;              // are we on the bottom (vs top)?
-} ncmenu;
-
 typedef struct ncprogbar {
   ncplane* ncp;
   double progress;          // on the range [0, 1]
@@ -260,18 +230,6 @@ typedef struct nctab {
   struct nctab* next;
 } nctab;
 
-typedef struct nctabbed {
-  ncplane* ncp;          // widget ncplane
-  ncplane* p;            // tab content ncplane
-  ncplane* hp;           // tab headers ncplane
-  // a doubly-linked circular list of tabs
-  nctab* leftmost;       // the tab most to the left
-  nctab* selected;       // the currently selected tab
-  int tabcount;          // tab separator (can be NULL)
-  int sepcols;           // separator with in columns
-  nctabbed_options opts; // copied in nctabbed_create()
-} nctabbed;
-
 // various moving parts within a notcurses context (and the user) might need to
 // access the stats object, so throw a lock on it. we don't want the lock in
 // the actual structure since (a) it's usually unnecessary and (b) it breaks
@@ -396,6 +354,8 @@ typedef struct blitterargs {
     struct {
       int colorregs;   // number of color registers
       sprixel* spx;    // sprixel object
+      int pxoffy;      // pixel y-offset within origin cell
+      int pxoffx;      // pixel x-offset within origin cell
     } pixel;           // for pixels
   } u;
 } blitterargs;
@@ -409,8 +369,8 @@ typedef int (*ncblitter)(struct ncplane* n, int linesize, const void* data,
 // a system for rendering RGBA pixels as text glyphs or sixel/kitty bitmaps
 struct blitset {
   ncblitter_e geom;
-  int width;        // number of input pixels per output cell, width
-  int height;       // number of input pixels per output cell, height
+  unsigned width;   // number of input pixels per output cell, width
+  unsigned height;  // number of input pixels per output cell, height
   // the EGCs which form the blitter. bits grow left to right, and then top to
   // bottom. the first character is always a space, the last a full block.
   const wchar_t* egcs;
@@ -689,12 +649,6 @@ void sprixel_movefrom(sprixel* s, int y, int x);
 void sprixel_debug(const sprixel* s, FILE* out);
 void sixelmap_free(struct sixelmap *s);
 
-// create an auxiliary vector suitable for a sprixcell, and zero it out. there
-// are two bytes per pixel in the cell. kitty uses only one (for an alpha
-// value). sixel uses both (for palette index, and transparency). FIXME fold
-// the transparency vector up into 1/8th as many bytes.
-uint8_t* sprixel_auxiliary_vector(const sprixel* s);
-
 // update any necessary cells underneath the sprixel pursuant to its removal.
 // for sixel, this *achieves* the removal, and is performed on every cell.
 // returns 1 if the graphic can be immediately freed (which is equivalent to
@@ -757,6 +711,25 @@ sprite_commit(tinfo* ti, fbuf* f, sprixel* s, unsigned forcescroll){
   return 0;
 }
 
+static inline void
+cleanup_tam(tament* tam, int ydim, int xdim){
+  for(int y = 0 ; y < ydim ; ++y){
+    for(int x = 0 ; x < xdim ; ++x){
+      free(tam[y * xdim + x].auxvector);
+      tam[y * xdim + x].auxvector = NULL;
+    }
+  }
+}
+
+static inline void
+destroy_tam(ncplane* p){
+  if(p->tam){
+    cleanup_tam(p->tam, p->leny, p->lenx);
+    free(p->tam);
+    p->tam = NULL;
+  }
+}
+
 static inline int
 sprite_rebuild(const notcurses* nc, sprixel* s, int ycell, int xcell){
   logdebug("rebuilding %d %d/%d\n", s->id, ycell, xcell);
@@ -767,7 +740,7 @@ sprite_rebuild(const notcurses* nc, sprixel* s, int ycell, int xcell){
   if(s->n->tam[idx].state == SPRIXCELL_ANNIHILATED_TRANS){
     s->n->tam[idx].state = SPRIXCELL_TRANSPARENT;
   }else if(s->n->tam[idx].state == SPRIXCELL_ANNIHILATED){
-    uint8_t* auxvec = s->n->tam[idx].auxvector;
+    uint8_t* auxvec = (uint8_t*)s->n->tam[idx].auxvector;
     assert(auxvec);
     // sets the new state itself
     ret = nc->tcache.pixel_rebuild(s, ycell, xcell, auxvec);
@@ -1033,7 +1006,7 @@ calc_gradient_channels(uint64_t* channels, uint64_t ul, uint64_t ur,
 }
 
 // ncdirect needs to "fake" an isolated ncplane as a drawing surface for
-// ncvisual_render(), and thus calls these low-level internal functions.
+// ncvisual_blit(), and thus calls these low-level internal functions.
 // they are not for general use -- check ncplane_new() and ncplane_destroy().
 ncplane* ncplane_new_internal(notcurses* nc, ncplane* n, const ncplane_options* nopts);
 
@@ -1044,9 +1017,9 @@ ALLOC char* ncplane_vprintf_prep(const char* format, va_list ap);
 
 // Resize the provided ncvisual to the specified 'rows' x 'cols', but do not
 // change the internals of the ncvisual. Uses oframe.
-int ncvisual_blit(struct ncvisual* ncv, int rows, int cols,
-                  ncplane* n, const struct blitset* bset,
-                  const blitterargs* bargs);
+int ncvisual_blit_internal(struct ncvisual* ncv, int rows, int cols,
+                           ncplane* n, const struct blitset* bset,
+                           const blitterargs* bargs);
 
 static inline int
 tty_emit(const char* seq, int fd){
@@ -1383,7 +1356,7 @@ plane_blit_sixel(sprixel* spx, fbuf* f, int leny, int lenx,
   if(n){
 //fprintf(stderr, "TAM WAS: %p NOW: %p\n", n->tam, tam);
     if(n->tam != tam){
-      free(n->tam);
+      destroy_tam(n);
     }
     n->tam = tam;
     n->sprite = spx;
@@ -1786,8 +1759,12 @@ emit_scrolls(const tinfo* ti, int count, fbuf* f){
 // replace or populate the TERM environment variable with 'termname'
 int putenv_term(const char* termname) __attribute__ ((nonnull (1)));
 
-#undef ALLOC
+// check environment for NOTCURSES_LOGLEVEL, and use it if defined.
+int set_loglevel_from_env(ncloglevel_e* loglevel)
+  __attribute__ ((nonnull (1)));
+
 #undef API
+#undef ALLOC
 
 #ifdef __cplusplus
 }
