@@ -347,8 +347,8 @@ kitty_anim_auxvec(int dimy, int dimx, int posy, int posx,
         }
       }
     }
+    ((uint8_t*)a)[slen - 1] = 0; // reset blitsource ownership
   }
-  ((uint8_t*)a)[slen - 1] = 0; // reset blitsource ownership
   return a;
 }
 
@@ -404,10 +404,10 @@ kitty_blit_wipe_selfref(sprixel* s, fbuf* f, int ycell, int xcell){
 // cell id with which we can delete it in O(1) for a rebuild. this
 // way, we needn't delete and redraw the entire sprixel.
 int kitty_wipe_animation(sprixel* s, int ycell, int xcell){
+  logdebug("wiping sprixel %u at %d/%d\n", s->id, ycell, xcell);
   if(init_sprixel_animation(s)){
     return -1;
   }
-  logdebug("wiping sprixel %u at %d/%d\n", s->id, ycell, xcell);
   fbuf* f = &s->glyph;
   if(kitty_blit_wipe_selfref(s, f, ycell, xcell) < 0){
     return -1;
@@ -445,9 +445,21 @@ sprixel* kitty_recycle(ncplane* n){
   return sprixel_alloc(&ncplane_notcurses_const(n)->tcache, n, dimy, dimx);
 }
 
+// for pre-animation kitty (NCPIXEL_KITTY_STATIC), we need a byte per pixel,
+// in which we stash the alpha.
+static inline uint8_t*
+kitty_auxiliary_vector(const sprixel* s){
+  int pixels = s->cellpxy * s->cellpxx;
+  uint8_t* ret = malloc(sizeof(*ret) * pixels);
+  if(ret){
+    memset(ret, 0, sizeof(*ret) * pixels);
+  }
+  return ret;
+}
+
 int kitty_wipe(sprixel* s, int ycell, int xcell){
 //fprintf(stderr, "NEW WIPE %d %d/%d\n", s->id, ycell, xcell);
-  uint8_t* auxvec = sprixel_auxiliary_vector(s);
+  uint8_t* auxvec = kitty_auxiliary_vector(s);
   if(auxvec == NULL){
     return -1;
   }
@@ -533,20 +545,20 @@ int kitty_wipe(sprixel* s, int ycell, int xcell){
 
 int kitty_commit(fbuf* f, sprixel* s, unsigned noscroll){
   loginfo("Committing Kitty graphic id %u\n", s->id);
-  fbuf_printf(f, "\e_Ga=p,i=%u,p=1,q=2%s\e\\", s->id, noscroll ? ",C=1" : "");
+  int i;
+  if(s->pxoffx || s->pxoffy){
+    i = fbuf_printf(f, "\e_Ga=p,i=%u,p=1,X=%u,Y=%u%s\e\\", s->id,
+                    s->pxoffx, s->pxoffy, noscroll ? ",C=1" : "");
+  }else{
+    i = fbuf_printf(f, "\e_Ga=p,i=%u,p=1,q=2%s\e\\", s->id, noscroll ? ",C=1" : "");
+  }
+  if(i < 0){
+    return -1;
+  }
   s->invalidated = SPRIXEL_QUIESCENT;
   return 0;
 }
 
-static void
-cleanup_tam(tament* tam, int ydim, int xdim){
-  for(int y = 0 ; y < ydim ; ++y){
-    for(int x = 0 ; x < xdim ; ++x){
-      free(tam[y * xdim + x].auxvector);
-    }
-  }
-}
-
 static inline void*
 zctx_origbuf(z_stream* zctx, int pixy, int pixx){
   size_t blen = pixx * pixy * 4;
@@ -730,9 +742,8 @@ static int
 write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
                  const uint32_t* data, const blitterargs* bargs,
                  tament* tam, int* parse_start, ncpixelimpl_e level){
-//fprintf(stderr, "drawing kitty %p\n", tam);
   if(linesize % sizeof(*data)){
-    logerror("Stride (%d) badly aligned\n", linesize);
+    logerror("stride (%d) badly aligned\n", linesize);
     return -1;
   }
   // we only deflate if we're using animation, since otherwise we need be able
@@ -805,7 +816,6 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
         if(translucent){
           ncpixel_set_a(&source[e], ncpixel_a(source[e]) / 2);
         }
-//fprintf(stderr, "%u/%u/%u -> %c%c%c%c %u %u %u %u\n", r, g, b, b64[0], b64[1], b64[2], b64[3], b64[0], b64[1], b64[2], b64[3]);
         int xcell = x / cdimx;
         int ycell = y / cdimy;
         int tyx = xcell + ycell * cols;
@@ -820,6 +830,7 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
                                     data, linesize, tam[tyx].auxvector,
                                     transcolor);
             if(tmp == NULL){
+              logerror("got a NULL auxvec at %d/%d\n", y, x);
               goto err;
             }
             tam[tyx].auxvector = tmp;
@@ -827,6 +838,7 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
             if(tam[tyx].auxvector == NULL){
               tam[tyx].auxvector = malloc(sizeof(tam[tyx].state));
               if(tam[tyx].auxvector == NULL){
+                logerror("got a NULL auxvec at %d\n", tyx);
                 goto err;
               }
             }
@@ -839,12 +851,12 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
             // out, to present a glyph "atop" it). we will continue to mark it
             // transparent, but we need to update the auxiliary vector.
             const int vyx = (y % cdimy) * cdimx + (x % cdimx);
-            tam[tyx].auxvector[vyx] = ncpixel_a(source[e]);
+            ((uint8_t*)tam[tyx].auxvector)[vyx] =  ncpixel_a(source[e]);
             wipe[e] = 1;
           }else if(level == NCPIXEL_KITTY_SELFREF){
             selfref_annihilated = true;
           }else{
-            tam[tyx].auxvector[s->cellpxx * s->cellpxy * 4] = 1;
+            ((uint8_t*)tam[tyx].auxvector)[s->cellpxx * s->cellpxy * 4] = 1;
             wipe[e] = 1;
           }
           if(rgba_trans_p(source[e], transcolor)){
@@ -852,16 +864,16 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
             if(x % cdimx == 0 && y % cdimy == 0){
               tam[tyx].state = SPRIXCELL_ANNIHILATED_TRANS;
               if(level == NCPIXEL_KITTY_SELFREF){
-                *tam[tyx].auxvector = SPRIXCELL_TRANSPARENT;
+                *(sprixcell_e*)tam[tyx].auxvector = SPRIXCELL_TRANSPARENT;
               }
             }else if(level == NCPIXEL_KITTY_SELFREF && tam[tyx].state == SPRIXCELL_ANNIHILATED_TRANS){
-                *tam[tyx].auxvector = SPRIXCELL_MIXED_KITTY;
+                *(sprixcell_e*)tam[tyx].auxvector = SPRIXCELL_MIXED_KITTY;
             }
           }else{
             if(x % cdimx == 0 && y % cdimy == 0 && level == NCPIXEL_KITTY_SELFREF){
-              *tam[tyx].auxvector = SPRIXCELL_OPAQUE_KITTY;
-            }else if(level == NCPIXEL_KITTY_SELFREF && *tam[tyx].auxvector == SPRIXCELL_TRANSPARENT){
-              *tam[tyx].auxvector = SPRIXCELL_MIXED_KITTY;
+              *(sprixcell_e*)tam[tyx].auxvector = SPRIXCELL_OPAQUE_KITTY;
+            }else if(level == NCPIXEL_KITTY_SELFREF && *(sprixcell_e*)tam[tyx].auxvector == SPRIXCELL_TRANSPARENT){
+              *(sprixcell_e*)tam[tyx].auxvector = SPRIXCELL_MIXED_KITTY;
             }
             tam[tyx].state = SPRIXCELL_ANNIHILATED;
           }
@@ -919,6 +931,7 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
   return 0;
 
 err:
+  logerror("failed blitting kitty graphics\n");
   cleanup_tam(tam, (leny + cdimy - 1) / cdimy, (lenx + cdimx - 1) / cdimx);
   destroy_deflator(animated, &zctx, leny, lenx);
   return -1;
@@ -947,11 +960,11 @@ int kitty_rebuild_selfref(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 }
 
 int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
+  logdebug("rebuilding sprixel %u %d at %d/%d\n", s->id, s->invalidated, ycell, xcell);
   if(init_sprixel_animation(s)){
     return -1;
   }
   fbuf* f = &s->glyph;
-  logdebug("rebuilding sprixel %u %d at %d/%d\n", s->id, s->invalidated, ycell, xcell);
   const int ystart = ycell * s->cellpxy;
   const int xstart = xcell * s->cellpxx;
   const int xlen = xstart + s->cellpxx > s->pixx ? s->pixx - xstart : s->cellpxx;
@@ -1052,49 +1065,34 @@ int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 static inline int
 kitty_blit_core(ncplane* n, int linesize, const void* data, int leny, int lenx,
                 const blitterargs* bargs, ncpixelimpl_e level){
-//fprintf(stderr, "IMAGE: start %p end %p\n", data, (const char*)data + leny * linesize);
   int cols = bargs->u.pixel.spx->dimx;
-  int rows = bargs->u.pixel.spx->dimy;
   sprixel* s = bargs->u.pixel.spx;
   if(init_sprixel_animation(s)){
     return -1;
   }
-  tament* tam = NULL;
-  bool reuse = false;
-  // if we have a sprixel attached to this plane, see if we can reuse it
-  // (we need the same dimensions) and thus immediately apply its T-A table.
-  if(n->tam){
-    if(n->leny == rows && n->lenx == cols){
-      tam = n->tam;
-      reuse = true;
-    }
-  }
   int parse_start = 0;
-  if(!reuse){
-    tam = malloc(sizeof(*tam) * rows * cols);
-    if(tam == NULL){
-      goto error;
-    }
-    memset(tam, 0, sizeof(*tam) * rows * cols);
-  }
   fbuf* f = &s->glyph;
+  int pxoffx = bargs->u.pixel.pxoffx;
+  int pxoffy = bargs->u.pixel.pxoffy;
   if(write_kitty_data(f, linesize, leny, lenx, cols, data,
-                      bargs, tam, &parse_start, level)){
+                      bargs, n->tam, &parse_start, level)){
     goto error;
   }
+  // FIXME need set pxoffx and pxoffy in sprixel
   if(level == NCPIXEL_KITTY_STATIC){
     s->animating = false;
   }
   // take ownership of |buf| and |tam| on success.
-  if(plane_blit_sixel(s, &s->glyph, leny, lenx, parse_start, tam, SPRIXEL_UNSEEN) < 0){
+  if(plane_blit_sixel(s, &s->glyph, leny + pxoffy, lenx + pxoffx, parse_start,
+                      n->tam, SPRIXEL_UNSEEN) < 0){
     goto error;
   }
+  s->pxoffx = pxoffx;
+  s->pxoffy = pxoffy;
   return 1;
 
 error:
-  if(!reuse){
-    free(tam);
-  }
+  cleanup_tam(n->tam, bargs->u.pixel.spx->dimy, bargs->u.pixel.spx->dimx);
   fbuf_free(&s->glyph);
   return -1;
 }
