@@ -3,8 +3,18 @@
 
 // auxvecs for framebuffer are 1B each for s->cellpxx * s->cellpxy elements,
 // and store the original alpha value.
+static inline uint8_t*
+fbcon_auxiliary_vector(const sprixel* s){
+  int pixels = s->cellpxy * s->cellpxx;
+  uint8_t* ret = malloc(sizeof(*ret) * pixels);
+  if(ret){
+    memset(ret, 0, sizeof(*ret) * pixels);
+  }
+  return ret;
+}
+
 int fbcon_wipe(sprixel* s, int ycell, int xcell){
-  uint8_t* auxvec = sprixel_auxiliary_vector(s);
+  uint8_t* auxvec = fbcon_auxiliary_vector(s);
   if(auxvec == NULL){
     return -1;
   }
@@ -32,8 +42,6 @@ int fbcon_wipe(sprixel* s, int ycell, int xcell){
 int fbcon_blit(struct ncplane* n, int linesize, const void* data,
                int leny, int lenx, const struct blitterargs* bargs){
   uint32_t transcolor = bargs->transcolor;
-  int cols = bargs->u.pixel.spx->dimx;
-  int rows = bargs->u.pixel.spx->dimy;
   sprixel* s = bargs->u.pixel.spx;
   int cdimx = s->cellpxx;
   int cdimy = s->cellpxy;
@@ -42,21 +50,6 @@ int fbcon_blit(struct ncplane* n, int linesize, const void* data,
   if(fbuf_reserve(&s->glyph, flen)){
     return -1;
   }
-  tament* tam = NULL;
-  bool reuse = false;
-  if(n->tam){
-    if(n->leny == rows && n->lenx == cols){
-      tam = n->tam;
-      reuse = true;
-    }
-  }
-  if(!reuse){
-    tam = malloc(sizeof(*tam) * rows * cols);
-    if(tam == NULL){
-      goto error;
-    }
-    memset(tam, 0, sizeof(*tam) * rows * cols);
-  }
   for(int l = 0 ; l < leny ; ++l){
     int ycell = l / cdimy;
     size_t soffset = l * linesize;
@@ -66,32 +59,32 @@ int fbcon_blit(struct ncplane* n, int linesize, const void* data,
     for(int c = 0 ; c < lenx ; ++c){
       int xcell = c / cdimx;
       int tyx = xcell + ycell * bargs->u.pixel.spx->dimx;
-      if(tam[tyx].state >= SPRIXCELL_ANNIHILATED){
+      if(n->tam[tyx].state >= SPRIXCELL_ANNIHILATED){
         if(rgba_trans_p(*(uint32_t*)src, transcolor)){
           ncpixel_set_a((uint32_t*)src, 0); // in case it was transcolor
           if(c % cdimx == 0 && l % cdimy == 0){
-            tam[tyx].state = SPRIXCELL_ANNIHILATED_TRANS;
+            n->tam[tyx].state = SPRIXCELL_ANNIHILATED_TRANS;
           }
         }else{
-          tam[tyx].state = SPRIXCELL_ANNIHILATED;
+          n->tam[tyx].state = SPRIXCELL_ANNIHILATED;
         }
         dst[3] = 0;
         const int vyx = (l % cdimy) * cdimx + (c % cdimx);
-        tam[tyx].auxvector[vyx] = src[3];
+        ((uint8_t*)n->tam[tyx].auxvector)[vyx] = src[3];
       }else{
         if(rgba_trans_p(*(uint32_t*)src, transcolor)){
           ncpixel_set_a((uint32_t*)src, 0); // in case it was transcolor
           if(c % cdimx == 0 && l % cdimy == 0){
-            tam[tyx].state = SPRIXCELL_TRANSPARENT;
-          }else if(tam[tyx].state == SPRIXCELL_OPAQUE_SIXEL){
-            tam[tyx].state = SPRIXCELL_MIXED_SIXEL;
+            n->tam[tyx].state = SPRIXCELL_TRANSPARENT;
+          }else if(n->tam[tyx].state == SPRIXCELL_OPAQUE_SIXEL){
+            n->tam[tyx].state = SPRIXCELL_MIXED_SIXEL;
           }
           dst[3] = 0;
         }else{
           if(c % cdimx == 0 && l % cdimy == 0){
-            tam[tyx].state = SPRIXCELL_OPAQUE_SIXEL;
-          }else if(tam[tyx].state == SPRIXCELL_TRANSPARENT){
-            tam[tyx].state = SPRIXCELL_MIXED_SIXEL;
+            n->tam[tyx].state = SPRIXCELL_OPAQUE_SIXEL;
+          }else if(n->tam[tyx].state == SPRIXCELL_TRANSPARENT){
+            n->tam[tyx].state = SPRIXCELL_MIXED_SIXEL;
           }
           memcpy(dst + 3, src + 3, 1);
         }
@@ -103,16 +96,13 @@ int fbcon_blit(struct ncplane* n, int linesize, const void* data,
       src += 4;
     }
   }
-  scrub_tam_boundaries(tam, leny, lenx, cdimy, cdimx);
-  if(plane_blit_sixel(s, &s->glyph, leny, lenx, 0, tam, SPRIXEL_INVALIDATED) < 0){
+  scrub_tam_boundaries(n->tam, leny, lenx, cdimy, cdimx);
+  if(plane_blit_sixel(s, &s->glyph, leny, lenx, 0, n->tam, SPRIXEL_INVALIDATED) < 0){
     goto error;
   }
   return 1;
 
 error:
-  if(!reuse){
-    free(tam);
-  }
   fbuf_free(&s->glyph);
   s->glyph.size = 0;
   return -1;
@@ -160,7 +150,6 @@ int fbcon_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
         }
       }
       s->glyph.buf[offset + 3] = auxvec[vyx];
-      offset += 4;
     }
   }
   s->n->tam[s->dimx * ycell + xcell].state = state;
@@ -639,8 +628,8 @@ reprogram_linux_font(tinfo* ti, int fd, struct console_font_op* cfo,
     logwarn("error reading Linux kernelfont (%s)\n", strerror(errno));
     return -1;
   }
-  loginfo("Kernel font size (glyphcount): %hu\n", cfo->charcount);
-  loginfo("Kernel font character geometry: %hux%hu\n", cfo->width, cfo->height);
+  loginfo("Kernel font size (glyphcount): %u\n", cfo->charcount);
+  loginfo("Kernel font character geometry: %ux%u\n", cfo->width, cfo->height);
   if(cfo->charcount > 512){
     logwarn("Warning: kernel returned excess charcount\n");
     return -1;
@@ -649,7 +638,7 @@ reprogram_linux_font(tinfo* ti, int fd, struct console_font_op* cfo,
     logwarn("error reading Linux unimap (%s)\n", strerror(errno));
     return -1;
   }
-  loginfo("Kernel Unimap size: %hu/%hu\n", map->entry_ct, USHRT_MAX);
+  loginfo("Kernel Unimap size: %u/%u\n", map->entry_ct, USHRT_MAX);
   // for certain sets of characters, we're not going to draw them in, but we
   // do want to ensure they map to something plausible...this doesn't reset
   // the framebuffer, even if we do some reprogramming.
