@@ -71,12 +71,12 @@ int reset_term_attributes(const tinfo* ti, fbuf* f){
 static int
 notcurses_stop_minimal(void* vnc){
   notcurses* nc = vnc;
+  int ret = 0;
+  ret |= drop_signals(nc);
   // collect output into the memstream buffer, and then dump it directly using
   // blocking_write(), to avoid problems with unreliable fflush().
   fbuf* f = &nc->rstate.f;
   fbuf_reset(f);
-  int ret = 0;
-  ret |= drop_signals(nc);
   // be sure to write the restoration sequences *prior* to running rmcup, as
   // they apply to the screen (alternate or otherwise) we're actually using.
   const char* esc;
@@ -100,20 +100,24 @@ notcurses_stop_minimal(void* vnc){
     if(nc->tcache.tpreserved){
       ret |= tcsetattr(nc->tcache.ttyfd, TCSAFLUSH, nc->tcache.tpreserved);
     }
+    // don't use use leave_alternate_screen() here; we need pop the keyboard
+    // whether we're in regular or altnerate screen, and we need it done
+    // before returning to the regular screen if we're in the alternate.
     if(nc->tcache.kbdlevel){
       if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
         ret = -1;
       }
     }
-    if((esc = get_escape(&nc->tcache, ESCAPE_RMCUP))){
-      if(sprite_clear_all(&nc->tcache, f)){ // send this to f
-        ret = -1;
-      }
-      if(tty_emit(esc, nc->tcache.ttyfd)){ // but this goes to ttyfd
-        ret = -1;
+    if(nc->tcache.in_alt_screen){
+      if((esc = get_escape(&nc->tcache, ESCAPE_RMCUP))){
+        if(tty_emit(esc, nc->tcache.ttyfd)){
+          ret = -1;
+        }
+        nc->tcache.in_alt_screen = 0;
       }
     }
   }
+  logdebug("reset terminal, returning %d\n", ret);
   return ret;
 }
 
@@ -360,15 +364,7 @@ void free_plane(ncplane* p){
     if(p->sprite){
       sprixel_hide(p->sprite);
     }
-    if(p->tam){
-      for(int y = 0 ; y < p->leny ; ++y){
-        for(int x = 0 ; x < p->lenx ; ++x){
-          free(p->tam[y * p->lenx + x].auxvector);
-          p->tam[y * p->lenx + x].auxvector = NULL;
-        }
-      }
-    }
-    free(p->tam);
+    destroy_tam(p);
     egcpool_dump(&p->pool);
     free(p->name);
     free(p->fb);
@@ -376,7 +372,9 @@ void free_plane(ncplane* p){
   }
 }
 
-// create a new ncpile. only call with pilelock held.
+// create a new ncpile. only call with pilelock held. the return value
+// was assigned to n->pile.
+__attribute__((malloc))
 static ncpile*
 make_ncpile(notcurses* nc, ncplane* n){
   ncpile* ret = malloc(sizeof(*ret));
@@ -395,7 +393,6 @@ make_ncpile(notcurses* nc, ncplane* n){
       ret->prev = ret;
       ret->next = ret;
     }
-    n->pile = ret;
     n->above = NULL;
     n->below = NULL;
     ret->dimy = 0;
@@ -405,6 +402,7 @@ make_ncpile(notcurses* nc, ncplane* n){
     ret->sprixelcache = NULL;
     ret->scrolls = 0;
   }
+  n->pile = ret;
   return ret;
 }
 
@@ -858,6 +856,7 @@ int ncplane_destroy(ncplane* ncp){
       ncp->bnext->bprev = ncp->bprev;
     }
   }else if(ncp->bnext){
+    //assert(ncp->boundto->blist == ncp);
     ncp->bnext->bprev = NULL;
   }
   // recursively reparent our children to the plane to which we are bound.
@@ -998,6 +997,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   ret->rstate.f.used = 0;
   ret->rstate.f.size = 0;
   ret->loglevel = opts->loglevel;
+  set_loglevel_from_env(&ret->loglevel);
   if(!(opts->flags & NCOPTION_INHIBIT_SETLOCALE)){
     init_lang();
   }
@@ -1075,7 +1075,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
                   &ret->rstate.logendy : &fakecursory;
   int* cursorx = opts->flags & NCOPTION_PRESERVE_CURSOR ?
                   &ret->rstate.logendx : &fakecursorx;
-  if(interrogate_terminfo(&ret->tcache, opts->termtype, ret->ttyfp, utf8,
+  if(interrogate_terminfo(&ret->tcache, ret->ttyfp, utf8,
                           opts->flags & NCOPTION_NO_ALTERNATE_SCREEN, 0,
                           opts->flags & NCOPTION_NO_FONT_CHANGES,
                           cursory, cursorx, &ret->stats,
@@ -2170,6 +2170,7 @@ void ncplane_erase(ncplane* n){
   loginfo("erasing plane\n");
   if(n->sprite){
     sprixel_hide(n->sprite);
+    destroy_tam(n);
   }
   // we must preserve the background, but a pure nccell_duplicate() would be
   // wiped out by the egcpool_dump(). do a duplication (to get the stylemask
@@ -2630,7 +2631,10 @@ splice_zaxis_recursive(ncplane* n, ncpile* p){
 }
 
 ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
-  if(n == ncplane_notcurses(n)->stdplane){
+  // ncplane_notcurses() goes through ncplane_pile(). since we're possibly
+  // destroying piles below, get the notcurses reference early on.
+  notcurses* nc = ncplane_notcurses(n);
+  if(n == nc->stdplane){
     return NULL; // can't reparent standard plane
   }
   if(ncplane_descendant_p(newparent, n)){
@@ -2649,9 +2653,6 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
   }
   n->bprev = NULL;
   n->bnext = NULL;
-  // ncplane_notcurses() goes through ncplane_pile(). since we're possibly
-  // destroying piles below, get the notcurses reference early on.
-  notcurses* nc = ncplane_notcurses(n);
   // if leaving a pile, extract n from the old zaxis, and also any sprixel
   sprixel* s = NULL;
   if(n == newparent || ncplane_pile(n) != ncplane_pile(newparent)){
@@ -2668,9 +2669,11 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
     if(ncplane_pile(n)->top == NULL){ // did we just empty our pile?
       ncpile_destroy(ncplane_pile(n));
     }
-    make_ncpile(ncplane_notcurses(n), n);
+    make_ncpile(nc, n);
     pthread_mutex_unlock(&nc->pilelock);
-    splice_zaxis_recursive(n, ncplane_pile(n));
+    if(ncplane_pile(n)){ // FIXME otherwise, we've got a problem...!
+      splice_zaxis_recursive(n, ncplane_pile(n));
+    }
   }else{ // establish ourselves as a sibling of new parent's children
     if( (n->bnext = newparent->blist) ){
       n->bnext->bprev = &n->bnext;
@@ -2896,8 +2899,8 @@ ncplane_as_rgba_internal(const ncplane* nc, ncblitter_e blit,
         ncchannels_bg_rgb8(channels, &br, &bb, &bg);
         ba = ncchannels_bg_alpha(channels);
         // handle each destination pixel from this cell
-        for(int py = 0 ; py < bset->height ; ++py){
-          for(int px = 0 ; px < bset->width ; ++px){
+        for(unsigned py = 0 ; py < bset->height ; ++py){
+          for(unsigned px = 0 ; px < bset->width ; ++px){
             uint32_t* p = &ret[(targy + py) * (lenx * bset->width) + (targx + px)];
             bool background = is_bg_p(idx, py, px, bset->width);
             if(background){
