@@ -1,5 +1,19 @@
 #include "internal.h"
 
+// internal ncselector item
+struct ncselector_int {
+  char* option;
+  char* desc;
+  size_t opcolumns;   // filled in by library
+  size_t desccolumns; // filled in by library
+};
+
+struct ncmselector_int {
+  char* option;
+  char* desc;
+  bool selected;
+};
+
 typedef struct ncselector {
   ncplane* ncp;                  // backing ncplane
   unsigned selected;             // index of selection
@@ -7,7 +21,7 @@ typedef struct ncselector {
   unsigned maxdisplay;           // max number of items to display, 0 -> no limit
   int longop;                    // columns occupied by longest option
   int longdesc;                  // columns occupied by longest description
-  struct ncselector_item* items; // list of items and descriptions, heap-copied
+  struct ncselector_int* items;  // list of items and descriptions, heap-copied
   unsigned itemcount;            // number of pairs in 'items'
   char* title;                   // can be NULL, in which case there's no riser
   int titlecols;                 // columns occupied by title
@@ -29,7 +43,7 @@ typedef struct ncmultiselector {
   unsigned startdisp;             // index of first option displayed
   unsigned maxdisplay;            // max number of items to display, 0 -> no limit
   int longitem;                   // columns occupied by longest item
-  struct ncmselector_item* items; // items, descriptions, and statuses, heap-copied
+  struct ncmselector_int* items;  // items, descriptions, and statuses, heap-copied
   unsigned itemcount;             // number of pairs in 'items'
   char* title;                    // can be NULL, in which case there's no riser
   int titlecols;                  // columns occupied by title
@@ -332,7 +346,7 @@ int ncselector_additem(ncselector* n, const struct ncselector_item* item){
   int origdimy, origdimx;
   ncselector_dim_yx(n, &origdimy, &origdimx);
   size_t newsize = sizeof(*n->items) * (n->itemcount + 1);
-  struct ncselector_item* items = realloc(n->items, newsize);
+  struct ncselector_int* items = realloc(n->items, newsize);
   if(!items){
     return -1;
   }
@@ -457,33 +471,7 @@ const char* ncselector_nextitem(ncselector* n){
 
 bool ncselector_offer_input(ncselector* n, const ncinput* nc){
   const int items_shown = ncplane_dim_y(n->ncp) - 4 - (n->title ? 2 : 0);
-  if(nc->id == NCKEY_UP){
-    ncselector_previtem(n);
-    return true;
-  }else if(nc->id == NCKEY_DOWN){
-    ncselector_nextitem(n);
-    return true;
-  }else if(nc->id == NCKEY_SCROLL_UP){
-    ncselector_previtem(n);
-    return true;
-  }else if(nc->id == NCKEY_SCROLL_DOWN){
-    ncselector_nextitem(n);
-    return true;
-  }else if(nc->id == NCKEY_PGDOWN){
-    if(items_shown > 0){
-      for(int i = 0 ; i < items_shown ; ++i){
-        ncselector_nextitem(n);
-      }
-    }
-    return true;
-  }else if(nc->id == NCKEY_PGUP){
-    if(items_shown > 0){
-      for(int i = 0 ; i < items_shown ; ++i){
-        ncselector_previtem(n);
-      }
-    }
-    return true;
-  }else if(nc->id == NCKEY_BUTTON1 && nc->evtype == NCTYPE_RELEASE){
+  if(nc->id == NCKEY_BUTTON1 && nc->evtype == NCTYPE_RELEASE){
     int y = nc->y, x = nc->x;
     if(!ncplane_translate_abs(n->ncp, &y, &x)){
       return false;
@@ -512,6 +500,34 @@ bool ncselector_offer_input(ncselector* n, const ncinput* nc){
       }
       return true;
     }
+  }else if(nc->evtype != NCTYPE_RELEASE){
+    if(nc->id == NCKEY_UP){
+      ncselector_previtem(n);
+      return true;
+    }else if(nc->id == NCKEY_DOWN){
+      ncselector_nextitem(n);
+      return true;
+    }else if(nc->id == NCKEY_SCROLL_UP){
+      ncselector_previtem(n);
+      return true;
+    }else if(nc->id == NCKEY_SCROLL_DOWN){
+      ncselector_nextitem(n);
+      return true;
+    }else if(nc->id == NCKEY_PGDOWN){
+      if(items_shown > 0){
+        for(int i = 0 ; i < items_shown ; ++i){
+          ncselector_nextitem(n);
+        }
+      }
+      return true;
+    }else if(nc->id == NCKEY_PGUP){
+      if(items_shown > 0){
+        for(int i = 0 ; i < items_shown ; ++i){
+          ncselector_previtem(n);
+        }
+      }
+      return true;
+    }
   }
   return false;
 }
@@ -732,37 +748,7 @@ const char* ncmultiselector_nextitem(ncmultiselector* n){
 
 bool ncmultiselector_offer_input(ncmultiselector* n, const ncinput* nc){
   const int items_shown = ncplane_dim_y(n->ncp) - 4 - (n->title ? 2 : 0);
-  if(nc->id == ' '){
-    n->items[n->current].selected = !n->items[n->current].selected;
-    ncmultiselector_draw(n);
-    return true;
-  }else if(nc->id == NCKEY_UP){
-    ncmultiselector_previtem(n);
-    return true;
-  }else if(nc->id == NCKEY_DOWN){
-    ncmultiselector_nextitem(n);
-    return true;
-  }else if(nc->id == NCKEY_PGDOWN){
-    if(items_shown > 0){
-      for(int i = 0 ; i < items_shown ; ++i){
-        ncmultiselector_nextitem(n);
-      }
-    }
-    return true;
-  }else if(nc->id == NCKEY_PGUP){
-    if(items_shown > 0){
-      for(int i = 0 ; i < items_shown ; ++i){
-        ncmultiselector_previtem(n);
-      }
-    }
-    return true;
-  }else if(nc->id == NCKEY_SCROLL_UP){
-    ncmultiselector_previtem(n);
-    return true;
-  }else if(nc->id == NCKEY_SCROLL_DOWN){
-    ncmultiselector_nextitem(n);
-    return true;
-  }else if(nc->id == NCKEY_BUTTON1 && nc->evtype == NCTYPE_RELEASE){
+  if(nc->id == NCKEY_BUTTON1 && nc->evtype == NCTYPE_RELEASE){
     int y = nc->y, x = nc->x;
     if(!ncplane_translate_abs(n->ncp, &y, &x)){
       return false;
@@ -791,6 +777,38 @@ bool ncmultiselector_offer_input(ncmultiselector* n, const ncinput* nc){
       }
       return true;
     }
+  }else if(nc->evtype != NCTYPE_RELEASE){
+    if(nc->id == ' '){
+      n->items[n->current].selected = !n->items[n->current].selected;
+      ncmultiselector_draw(n);
+      return true;
+    }else if(nc->id == NCKEY_UP){
+      ncmultiselector_previtem(n);
+      return true;
+    }else if(nc->id == NCKEY_DOWN){
+      ncmultiselector_nextitem(n);
+      return true;
+    }else if(nc->id == NCKEY_PGDOWN){
+      if(items_shown > 0){
+        for(int i = 0 ; i < items_shown ; ++i){
+          ncmultiselector_nextitem(n);
+        }
+      }
+      return true;
+    }else if(nc->id == NCKEY_PGUP){
+      if(items_shown > 0){
+        for(int i = 0 ; i < items_shown ; ++i){
+          ncmultiselector_previtem(n);
+        }
+      }
+      return true;
+    }else if(nc->id == NCKEY_SCROLL_UP){
+      ncmultiselector_previtem(n);
+      return true;
+    }else if(nc->id == NCKEY_SCROLL_DOWN){
+      ncmultiselector_nextitem(n);
+      return true;
+    }
   }
   return false;
 }
