@@ -20,6 +20,21 @@ sixelcount(int dimy, int dimx){
   return (dimy + 5) / 6 * dimx;
 }
 
+// create an auxiliary vector suitable for a Sixel sprixcell, and zero it out.
+// there are two bytes per pixel in the cell: a contiguous set of palette
+// indices (yet another reason that we work with 256 colors), and a contiguous
+// set of two-value transparencies (these could be folded down to bits from
+// bytes, saving 7/8 of the space FIXME).
+static inline uint8_t*
+sixel_auxiliary_vector(const sprixel* s){
+  int pixels = s->cellpxy * s->cellpxx;
+  uint8_t* ret = malloc(sizeof(*ret) * pixels * 2);
+  if(ret){
+    memset(ret, 0, sizeof(*ret) * pixels);
+  }
+  return ret;
+}
+
 // we keep a color-indexed set of sixels (a single-row column of six pixels,
 // encoded as a byte) across the life of the sprixel. This provides a good
 // combination of easy-to-edit (for wipes and restores) -- you can index by
@@ -210,7 +225,7 @@ wipe_color(sixelmap* smap, int color, int sband, int eband,
 // redrawn, it's redrawn using P2=1.
 int sixel_wipe(sprixel* s, int ycell, int xcell){
 //fprintf(stderr, "WIPING %d/%d\n", ycell, xcell);
-  uint8_t* auxvec = sprixel_auxiliary_vector(s);
+  uint8_t* auxvec = sixel_auxiliary_vector(s);
   if(auxvec == NULL){
     return -1;
   }
@@ -373,9 +388,11 @@ void sixel_refresh(const ncpile* p, sprixel* s){
       int idx = y * s->dimx + x;
       if(s->needs_refresh[idx]){
         const int xx = absx + x;
-        int ridx = yy * p->dimx + xx;
-        struct crender *r = &p->crender[ridx];
-        r->s.damaged = 1;
+        if(xx < p->dimx && yy < p->dimy){
+          int ridx = yy * p->dimx + xx;
+          struct crender *r = &p->crender[ridx];
+          r->s.damaged = 1;
+        }
       }
     }
   }
@@ -911,38 +928,16 @@ int sixel_blit(ncplane* n, int linesize, const void* data, int leny, int lenx,
   memset(stable.deets, 0, sizeof(*stable.deets) * colorregs);
   int cols = bargs->u.pixel.spx->dimx;
   int rows = bargs->u.pixel.spx->dimy;
-  tament* tam = NULL;
-  bool reuse = false;
-  // if we have a sprixel attached to this plane, see if we can reuse it
-  // (we need the same dimensions) and thus immediately apply its T-A table.
-  if(n->tam){
-    if(n->leny == rows && n->lenx == cols){
-      tam = n->tam;
-      reuse = true;
-    }
+  typeof(bargs->u.pixel.spx->needs_refresh) rmatrix;
+  rmatrix = malloc(sizeof(*rmatrix) * rows * cols);
+  if(rmatrix == NULL){
+    sixelmap_free(stable.map);
+    free(stable.deets);
+    return -1;
   }
-  if(!reuse){
-    tam = malloc(sizeof(*tam) * rows * cols);
-    if(tam == NULL){
-      sixelmap_free(stable.map);
-      free(stable.deets);
-      return -1;
-    }
-    memset(tam, 0, sizeof(*tam) * rows * cols);
-  }else{
-    typeof(bargs->u.pixel.spx->needs_refresh) rmatrix;
-    rmatrix = malloc(sizeof(*rmatrix) * rows * cols);
-    if(rmatrix == NULL){
-      sixelmap_free(stable.map);
-      free(stable.deets);
-      return -1;
-    }
-    bargs->u.pixel.spx->needs_refresh = rmatrix;
-  }
-  if(extract_color_table(data, linesize, cols, leny, lenx, &stable, tam, bargs)){
-    if(!reuse){
-      free(tam);
-    }
+  bargs->u.pixel.spx->needs_refresh = rmatrix;
+  assert(n->tam);
+  if(extract_color_table(data, linesize, cols, leny, lenx, &stable, n->tam, bargs)){
     free(bargs->u.pixel.spx->needs_refresh);
     sixelmap_free(stable.map);
     free(stable.deets);
@@ -950,7 +945,7 @@ int sixel_blit(ncplane* n, int linesize, const void* data, int leny, int lenx,
   }
   refine_color_table(data, linesize, bargs->begy, bargs->begx, leny, lenx, &stable);
   // takes ownership of sixelmap on success
-  int r = sixel_blit_inner(leny, lenx, &stable, bargs->u.pixel.spx, tam);
+  int r = sixel_blit_inner(leny, lenx, &stable, bargs->u.pixel.spx, n->tam);
   if(r < 0){
     sixelmap_free(stable.map);
   }
