@@ -123,7 +123,7 @@ typedef enum {
 // reconstruction of annihilated cells, valid only for SPRIXCELL_ANNIHILATED.
 typedef struct tament {
   sprixcell_e state;
-  uint8_t* auxvector; // palette entries for sixel, alphas for kitty
+  void* auxvector; // palette entries for sixel, alphas for kitty
 } tament;
 
 // a sprixel represents a bitmap, using whatever local protocol is available.
@@ -150,6 +150,7 @@ typedef struct sprixel {
   int movedfromx;       // so that we can damage old cells when redrawn
   // only used for kitty-based sprixels
   int parse_start;      // where to start parsing for cell wipes
+  int pxoffy, pxoffx;   // X and Y parameters to display command
   // only used for sixel-based sprixels
   unsigned char* needs_refresh; // one per cell, whether new frame needs damage
   struct sixelmap* smap;  // copy of palette indices + transparency bits
@@ -157,6 +158,16 @@ typedef struct sprixel {
   bool animating;        // do we have an active animation?
 } sprixel;
 
+static inline tament*
+create_tam(int rows, int cols){
+  // need cast for c++ callers
+  tament* tam = (tament*)malloc(sizeof(*tam) * rows * cols);
+  if(tam){
+    memset(tam, 0, sizeof(*tam) * rows * cols);
+  }
+  return tam;
+}
+
 int sixel_wipe(sprixel* s, int ycell, int xcell);
 // nulls out a cell from a kitty bitmap via changing the alpha value
 // throughout to 0. the same trick doesn't work on sixel, but there we
