@@ -174,7 +174,7 @@ void summarize_stats(notcurses* nc){
     qprefix(stats->render_min_ns, NANOSECS_IN_SEC, minbuf, 0);
     qprefix(stats->render_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
     qprefix(stats->render_ns / stats->renders, NANOSECS_IN_SEC, avgbuf, 0);
-    fprintf(stderr, "%s%"PRIu64" render%s, %ss (%ss min, %ss avg, %ss max)\n",
+    fprintf(stderr, "%s%"PRIu64" render%s, %ss (%ss min, %ss avg, %ss max)" NL,
             clreol, stats->renders, stats->renders == 1 ? "" : "s",
             totalbuf, minbuf, avgbuf, maxbuf);
   }
@@ -184,7 +184,7 @@ void summarize_stats(notcurses* nc){
     qprefix(stats->raster_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
     qprefix(stats->raster_ns / (stats->writeouts + stats->failed_writeouts),
             NANOSECS_IN_SEC, avgbuf, 0);
-    fprintf(stderr, "%s%"PRIu64" raster%s, %ss (%ss min, %ss avg, %ss max)\n",
+    fprintf(stderr, "%s%"PRIu64" raster%s, %ss (%ss min, %ss avg, %ss max)" NL,
             clreol, stats->writeouts, stats->writeouts == 1 ? "" : "s",
             totalbuf, minbuf, avgbuf, maxbuf);
     qprefix(stats->writeout_ns, NANOSECS_IN_SEC, totalbuf, 0);
@@ -193,7 +193,7 @@ void summarize_stats(notcurses* nc){
     qprefix(stats->writeout_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
     qprefix(stats->writeouts ? stats->writeout_ns / stats->writeouts : 0,
             NANOSECS_IN_SEC, avgbuf, 0);
-    fprintf(stderr, "%s%"PRIu64" write%s, %ss (%ss min, %ss avg, %ss max)\n",
+    fprintf(stderr, "%s%"PRIu64" write%s, %ss (%ss min, %ss avg, %ss max)" NL,
             clreol, stats->writeouts, stats->writeouts == 1 ? "" : "s",
             totalbuf, minbuf, avgbuf, maxbuf);
   }
@@ -203,24 +203,24 @@ void summarize_stats(notcurses* nc){
             1, minbuf, 1),
     bprefix(stats->renders ? stats->render_bytes / stats->renders : 0, 1, avgbuf, 1);
     bprefix(stats->render_max_bytes, 1, maxbuf, 1),
-    fprintf(stderr, "%s%sB (%sB min, %sB avg, %sB max) %"PRIu64" input%s\n",
+    fprintf(stderr, "%s%sB (%sB min, %sB avg, %sB max) %"PRIu64" input%s" NL,
             clreol, totalbuf, minbuf, avgbuf, maxbuf,
             stats->input_events,
             stats->input_events == 1 ? "" : "s");
   }
-  fprintf(stderr, "%s%"PRIu64" failed render%s, %"PRIu64" failed raster%s, %"PRIu64" refresh%s, %"PRIu64" input error%s\n",
+  fprintf(stderr, "%s%"PRIu64" failed render%s, %"PRIu64" failed raster%s, %"PRIu64" refresh%s, %"PRIu64" input error%s" NL,
           clreol, stats->failed_renders, stats->failed_renders == 1 ? "" : "s",
           stats->failed_writeouts, stats->failed_writeouts == 1 ? "" : "s",
           stats->refreshes, stats->refreshes == 1 ? "" : "es",
           stats->input_errors, stats->input_errors == 1 ? "" : "s");
-  fprintf(stderr, "%sRGB emits:elides: def %"PRIu64":%"PRIu64" fg %"PRIu64":%"PRIu64" bg %"PRIu64":%"PRIu64"\n",
+  fprintf(stderr, "%sRGB emits:elides: def %"PRIu64":%"PRIu64" fg %"PRIu64":%"PRIu64" bg %"PRIu64":%"PRIu64"" NL,
           clreol, stats->defaultemissions,
           stats->defaultelisions,
           stats->fgemissions,
           stats->fgelisions,
           stats->bgemissions,
           stats->bgelisions);
-  fprintf(stderr, "%sCell emits:elides: %"PRIu64":%"PRIu64" (%.2f%%) %.2f%% %.2f%% %.2f%%\n",
+  fprintf(stderr, "%sCell emits:elides: %"PRIu64":%"PRIu64" (%.2f%%) %.2f%% %.2f%% %.2f%%" NL,
           clreol, stats->cellemissions, stats->cellelisions,
           (stats->cellemissions + stats->cellelisions) == 0 ? 0 :
           (stats->cellelisions * 100.0) / (stats->cellemissions + stats->cellelisions),
@@ -231,7 +231,7 @@ void summarize_stats(notcurses* nc){
           (stats->bgemissions + stats->bgelisions) == 0 ? 0 :
           (stats->bgelisions * 100.0) / (stats->bgemissions + stats->bgelisions));
   bprefix(stats->sprixelbytes, 1, totalbuf, 1);
-  fprintf(stderr, "%sBitmap emits:elides: %"PRIu64":%"PRIu64" (%.2f%%) %sB (%.2f%%) SuM: %"PRIu64" (%.2f%%)\n",
+  fprintf(stderr, "%sBitmap emits:elides: %"PRIu64":%"PRIu64" (%.2f%%) %sB (%.2f%%) SuM: %"PRIu64" (%.2f%%)" NL,
           clreol, stats->sprixelemissions, stats->sprixelelisions,
           (stats->sprixelemissions + stats->sprixelelisions) == 0 ? 0 :
           (stats->sprixelelisions * 100.0) / (stats->sprixelemissions + stats->sprixelelisions),
