@@ -179,11 +179,11 @@ compare_versions(const char* restrict v1, const char* restrict v2){
   if(v1 == NULL){
     return -1;
   }
-  char* v1e;
-  char* v2e;
+  const char* v1e = v1;
+  const char* v2e = v2;
   while(*v1 && *v2){
-    long v1v = strtol(v1, &v1e, 10);
-    long v2v = strtol(v2, &v2e, 10);
+    long v1v = strtol(v1, (char **)&v1e, 10);
+    long v2v = strtol(v2, (char **)&v2e, 10);
     if(v1e == v1 && v2e == v2){ // both are done
       return 0;
     }else if(v1e == v1){ // first is done
@@ -208,6 +208,9 @@ compare_versions(const char* restrict v1, const char* restrict v2){
     v1 = v1e + 1;
     v2 = v2e + 1;
   }
+  if(*v1e == *v2e){
+    return 0;
+  }
   // can only get out here if at least one was not a period
   if(*v1e == '.'){
     return 1;
@@ -215,6 +218,12 @@ compare_versions(const char* restrict v1, const char* restrict v2){
   if(*v2e == '.'){
     return -1;
   }
+  if(!*v1e){
+    return -1;
+  }
+  if(!*v2e){
+    return 1;
+  }
   return 0;
 }
 
@@ -318,10 +327,9 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // which can be identified directly, sans queries.
 #define KITTYQUERY "\x1b_Gi=1,a=q;\x1b\\"
 
-// request kitty keyboard protocol through level 1, first pushing current.
+// request kitty keyboard protocol features 1, 2, and 8, first pushing current.
 // see https://sw.kovidgoyal.net/kitty/keyboard-protocol/#progressive-enhancement
-// FIXME go to level 11, but first need handle all functional keys
-#define KBDSUPPORT "\x1b[=3u"
+#define KBDSUPPORT "\x1b[=11u"
 
 // the kitty keyboard protocol allows unambiguous, complete identification of
 // input events. this queries for the level of support. we want to do this
@@ -342,6 +350,9 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
                   SECDEVATTR
 
 // query background, replies in X color https://www.x.org/releases/X11R7.7/doc/man/man7/X.7.xhtml#heading11
+// GNU screen passes this on to the underlying terminal rather than answering
+// itself, unlike most other queries, so send this first since it will take
+// longer to be answered.
 #define CSI_BGQ "\x1b]11;?\e\\"
 
 // FIXME ought be using the u7 terminfo string here, if it exists. the great
@@ -369,12 +380,9 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 #define GEOMCELL "\x1b[18t"
 
 #define DIRECTIVES CSI_BGQ \
+                   KBDQUERY \
                    SUMQUERY \
                    "\x1b[?1;3;256S" /* try to set 256 cregs */ \
-                   KKEYBOARD_PUSH \
-                   XTMODKEYS \
-                   KBDSUPPORT \
-                   KBDQUERY \
                    KITTYQUERY \
                    CREGSXTSM \
                    GEOMXTSM \
@@ -382,11 +390,16 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
                    GEOMCELL \
                    PRIDEVATTR
 
+// written whenever we switch between standard and alternate screen, or upon
+// startup (that's an entry into a screen! presumably the standard one).
+#define KBDENTER KKEYBOARD_PUSH XTMODKEYS KBDSUPPORT
+#define KBDLEAVE KKEYBOARD_POP // FIXME undo XTMODKEYS!
+
 // enter the alternate screen (smcup). we could technically get this from
 // terminfo, but everyone who supports it supports it the same way, and we
 // need to send it before our other directives if we're going to use it.
-#define SMCUP "\x1b[?1049h"
-#define RMCUP "\x1b[?1049l"
+#define SMCUP "\x1b[?1049h" KBDENTER
+#define RMCUP KBDLEAVE "\x1b[?1049l"
 
 // we send an XTSMGRAPHICS to set up 256 color registers (the most we can
 // currently take advantage of; we need at least 64 to use sixel at all).
@@ -398,9 +411,9 @@ send_initial_queries(int fd, bool minimal, bool noaltscreen){
   const char *queries;
   if(noaltscreen){
     if(minimal){
-      queries = DSRCPR DIRECTIVES;
+      queries = KBDENTER DSRCPR DIRECTIVES;
     }else{
-      queries = DSRCPR IDQUERIES DIRECTIVES;
+      queries = KBDENTER DSRCPR IDQUERIES DIRECTIVES;
     }
   }else{
     if(minimal){
@@ -417,6 +430,40 @@ send_initial_queries(int fd, bool minimal, bool noaltscreen){
   return 0;
 }
 
+int enter_alternate_screen(FILE* fp, tinfo* ti, bool flush){
+  if(ti->in_alt_screen){
+    return 0;
+  }
+  const char* smcup = get_escape(ti, ESCAPE_SMCUP);
+  if(smcup == NULL){
+    logerror("alternate screen is unavailable");
+    return -1;
+  }
+  if(term_emit(smcup, fp, false) || term_emit(KBDENTER, fp, flush)){
+    return -1;
+  }
+  ti->in_alt_screen = true;
+  return 0;
+}
+
+int leave_alternate_screen(FILE* fp, tinfo* ti){
+  if(!ti->in_alt_screen){
+    return 0;
+  }
+  const char* rmcup = get_escape(ti, ESCAPE_RMCUP);
+  if(rmcup == NULL){
+    logerror("can't leave alternate screen");
+    return -1;
+  }
+  if(term_emit(KBDLEAVE, fp, false) ||
+     term_emit(rmcup, fp, false) ||
+     term_emit(KBDENTER, fp, true)){
+    return -1;
+  }
+  ti->in_alt_screen = false;
+  return 0;
+}
+
 // if we get a response to the standard cursor locator escape, we know this
 // terminal supports it, hah.
 static int
@@ -502,6 +549,11 @@ static int
 apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm,
                       size_t* tablelen, size_t* tableused, bool* invertsixel,
                       unsigned nonewfonts){
+#ifdef __MINGW64__
+  if(qterm == TERMINAL_UNKNOWN){
+    qterm = TERMINAL_MSTERMINAL;
+  }
+#endif
   if(!termname){
     // setupterm interprets a missing/empty TERM variable as the special value “unknown”.
     termname = ti->termname ? ti->termname : "unknown";
@@ -564,6 +616,11 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
   }else if(qterm == TERMINAL_TMUX){
     termname = "tmux";
     // FIXME what, oh what to do with tmux?
+  }else if(qterm == TERMINAL_GNUSCREEN){
+    termname = "GNU screen";
+    if(compare_versions(ti->termversion, "5.0") < 0){
+      ti->caps.rgb = false;
+    }
   }else if(qterm == TERMINAL_MLTERM){
     termname = "MLterm";
     ti->caps.quadrants = true; // good caps.quadrants, no caps.sextants as of 3.9.0
@@ -579,14 +636,14 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     }
   }else if(qterm == TERMINAL_XTERM){
     termname = "XTerm";
+    if(compare_versions(ti->termversion, "369") < 0){
+      *invertsixel = true; // xterm 369 inverted DECSDM
+    }
     // xterm 357 added color palette escapes XT{PUSH,POP,REPORT}COLORS
     if(compare_versions(ti->termversion, "357") >= 0){
       if(add_pushcolors_escapes(ti, tablelen, tableused)){
         return -1;
       }
-      if(compare_versions(ti->termversion, "359") < 0){
-        *invertsixel = true;
-      }
     }
   }else if(qterm == TERMINAL_MINTTY){
     termname = "MinTTY";
@@ -597,9 +654,13 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       *invertsixel = true;
     }
     ti->bce = true;
+  }else if(qterm == TERMINAL_MSTERMINAL){
+    termname = "Windows Terminal";
+    ti->caps.rgb = true;
   }else if(qterm == TERMINAL_CONTOUR){
     termname = "Contour";
     ti->caps.quadrants = true;
+    ti->caps.sextants = true;
     ti->caps.rgb = true;
     *invertsixel = true;
   }else if(qterm == TERMINAL_ITERM){
@@ -717,11 +778,12 @@ macos_early_matches(void){
 // Device Attributes, allowing us to get a negative response if our queries
 // aren't supported by the terminal. we fire it off early because we have a
 // full round trip before getting the reply, which is likely to pace init.
-int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned utf8,
+int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
                          unsigned noaltscreen, unsigned nocbreak, unsigned nonewfonts,
                          int* cursor_y, int* cursor_x, ncsharedstats* stats,
                          int lmargin, int tmargin, int rmargin, int bmargin,
                          unsigned draininput){
+  const char* termtype = getenv("TERM");
   int foolcursor_x, foolcursor_y;
   if(!cursor_x){
     cursor_x = &foolcursor_x;
@@ -732,6 +794,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   *cursor_x = *cursor_y = -1;
   memset(ti, 0, sizeof(*ti));
   ti->bg_collides_default = 0xfe000000;
+  ti->kbdlevel = UINT_MAX; // see comment in tinfo definition
   ti->qterm = TERMINAL_UNKNOWN;
   // we don't need a controlling tty for everything we do; allow a failure here
   ti->ttyfd = get_tty_fd(out);
@@ -745,11 +808,12 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   if(termtype){
     logwarn("termtype (%s) ignored on windows\n", termtype);
   }
-  if(prepare_windows_terminal(ti, &tablelen, &tableused) == 0){
-    ti->qterm = TERMINAL_MSTERMINAL;
-    if(cursor_y && cursor_x){
-      locate_cursor(ti, cursor_y, cursor_x);
-    }
+  if(prepare_windows_terminal(ti, &tablelen, &tableused)){
+    logpanic("failed opening Windows ConPTY\n");
+    return -1;
+  }
+  if(cursor_y && cursor_x){
+    locate_cursor(ti, cursor_y, cursor_x);
   }
 #elif defined(__linux__)
   ti->linux_fb_fd = -1;
@@ -982,6 +1046,8 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
         goto err;
       }
     }
+  }else{
+    ti->kbdlevel = 0; // confirmed no support, don't bother popping
   }
   if(*cursor_x >= 0 && *cursor_y >= 0){
     if(add_u7_escape(ti, &tablelen, &tableused)){
@@ -1048,18 +1114,16 @@ char* termdesc_longterm(const tinfo* ti){
 // user input looking at infd.
 int locate_cursor(tinfo* ti, int* cursor_y, int* cursor_x){
 #ifdef __MINGW64__
-  if(ti->qterm == TERMINAL_MSTERMINAL){
-    if(ti->outhandle){
-      CONSOLE_SCREEN_BUFFER_INFO conbuf;
-      if(!GetConsoleScreenBufferInfo(ti->outhandle, &conbuf)){
-        logerror("couldn't get cursor info\n");
-        return -1;
-      }
-      *cursor_y = conbuf.dwCursorPosition.Y;
-      *cursor_x = conbuf.dwCursorPosition.X;
-      loginfo("got a report from y=%d x=%d\n", *cursor_y, *cursor_x);
-      return 0;
+  if(ti->outhandle){
+    CONSOLE_SCREEN_BUFFER_INFO conbuf;
+    if(!GetConsoleScreenBufferInfo(ti->outhandle, &conbuf)){
+      logerror("couldn't get cursor info\n");
+      return -1;
     }
+    *cursor_y = conbuf.dwCursorPosition.Y;
+    *cursor_x = conbuf.dwCursorPosition.X;
+    loginfo("got a report from y=%d x=%d\n", *cursor_y, *cursor_x);
+    return 0;
   }
 #endif
   const char* u7 = get_escape(ti, ESCAPE_U7);
@@ -1146,7 +1210,7 @@ int putenv_term(const char* termname){
   }else{
     loginfo("provided %s value %s\n", ENVVAR, termname);
   }
-  if(strcmp(oldterm, termname) == 0){
+  if(oldterm && strcmp(oldterm, termname) == 0){
     return 0;
   }
   char* buf = malloc(strlen(termname) + strlen(ENVVAR) + 1);
