@@ -32,7 +32,7 @@ void ncvisual_printbanner(fbuf* f){
 
 // you need an actual multimedia implementation for functions which work with
 // codecs, including ncvisual_decode(), ncvisual_decode_loop(),
-// ncvisual_from_file(), ncvisual_stream(), and ncvisual_subtitle().
+// ncvisual_from_file(), ncvisual_stream(), and ncvisual_subtitle_plane().
 int ncvisual_decode(ncvisual* nc){
   if(!visual_implementation.visual_decode){
     return -1;
@@ -75,8 +75,8 @@ char* ncvisual_subtitle(const ncvisual* ncv){
   return NULL;
 }
 
-int ncvisual_blit(ncvisual* ncv, int rows, int cols, ncplane* n,
-                  const struct blitset* bset, const blitterargs* barg){
+int ncvisual_blit_internal(ncvisual* ncv, int rows, int cols, ncplane* n,
+                           const struct blitset* bset, const blitterargs* barg){
   if(!(barg->flags & NCVISUAL_OPTION_NOINTERPOLATE)){
     if(visual_implementation.visual_blit){
       if(visual_implementation.visual_blit(ncv, rows, cols, n, bset, barg) < 0){
@@ -195,35 +195,54 @@ int ncvisual_blitset_geom(const notcurses* nc, const tinfo* tcache,
   if(blitter){
     *blitter = bset;
   }
-  if(bset->geom == NCBLIT_PIXEL && vopts){
-    if(vopts->n){
-      if(vopts->n == notcurses_stdplane_const(nc)){
-        if(!(vopts->flags & NCVISUAL_OPTION_CHILDPLANE)){
-          logerror("Won't blit bitmaps to the standard plane\n");
+  if(vopts){
+    if(bset->geom == NCBLIT_PIXEL){
+      if(vopts->n){
+        if(vopts->n == notcurses_stdplane_const(nc)){
+          if(!(vopts->flags & NCVISUAL_OPTION_CHILDPLANE)){
+            logerror("Won't blit bitmaps to the standard plane\n");
+            return -1;
+          }
+        }
+        if(vopts->y && !(vopts->flags & (NCVISUAL_OPTION_VERALIGNED | NCVISUAL_OPTION_CHILDPLANE))){
+          logerror("Non-origin y placement %d for sprixel\n", vopts->y);
           return -1;
         }
+        if(vopts->x && !(vopts->flags & (NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_CHILDPLANE))){
+          logerror("Non-origin x placement %d for sprixel\n", vopts->x);
+          return -1;
+        }
+        // FIXME clamp to sprixel limits
+        if(vopts->scaling == NCSCALE_NONE || vopts->scaling == NCSCALE_NONE_HIRES){
+          if(vopts->pxoffy >= nc->tcache.cellpixy){
+            logerror("pixel y-offset %d too tall for cell %d\n", vopts->pxoffy, nc->tcache.cellpixy);
+            return -1;
+          }
+          int rows = ((*leny + nc->tcache.cellpixy - 1) / nc->tcache.cellpixy)
+                      + !!vopts->pxoffy;
+          if(rows > ncplane_dim_y(vopts->n)){
+            logerror("sprixel too tall %d for plane %d\n",
+                     *leny + vopts->pxoffy, ncplane_dim_y(vopts->n) * nc->tcache.cellpixy);
+            return -1;
+          }
+          if(vopts->pxoffx >= nc->tcache.cellpixx){
+            logerror("pixel x-offset %d too wide for cell %d\n", vopts->pxoffx, nc->tcache.cellpixx);
+            return -1;
+          }
+          int cols = ((*lenx + nc->tcache.cellpixx - 1) / nc->tcache.cellpixx)
+                      + !!vopts->pxoffx;
+          if(cols > ncplane_dim_x(vopts->n)){
+            logerror("sprixel too wide %d for plane %d\n",
+                     *lenx + vopts->pxoffx, ncplane_dim_x(vopts->n) * nc->tcache.cellpixx);
+            return -1;
+          }
+        }
       }
-      if(vopts->y && !(vopts->flags & (NCVISUAL_OPTION_VERALIGNED | NCVISUAL_OPTION_CHILDPLANE))){
-        logerror("Non-origin y placement %d for sprixel\n", vopts->y);
+    }else{
+      if(vopts->pxoffx || vopts->pxoffy){
+        logerror("pixel offsets cannot be used with cell blitting\n");
         return -1;
       }
-      if(vopts->x && !(vopts->flags & (NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_CHILDPLANE))){
-        logerror("Non-origin x placement %d for sprixel\n", vopts->x);
-        return -1;
-      }
-      // FIXME clamp to sprixel limits
-      if(vopts->scaling == NCSCALE_NONE || vopts->scaling == NCSCALE_NONE_HIRES){
-        int rows = (*leny + nc->tcache.cellpixy - 1) / nc->tcache.cellpixy;
-        if(rows > ncplane_dim_y(vopts->n)){
-          logerror("Sprixel too tall %d for plane %d\n", *leny, ncplane_dim_y(vopts->n) * nc->tcache.cellpixy);
-          return -1;
-        }
-        int cols = (*lenx + nc->tcache.cellpixx - 1) / nc->tcache.cellpixx;
-        if(cols > ncplane_dim_x(vopts->n)){
-          logerror("Sprixel too wide %d for plane %d\n", *lenx, ncplane_dim_x(vopts->n) * nc->tcache.cellpixx);
-          return -1;
-        }
-      }
     }
   }
   if(n){
@@ -790,14 +809,15 @@ ncplane* ncvisual_render_cells(notcurses* nc, ncvisual* ncv, const struct blitse
   ncplane* createdn = NULL;
 //fprintf(stderr, "INPUT N: %p\n", n);
   if(n == NULL || (flags & NCVISUAL_OPTION_CHILDPLANE)){ // create plane
-    if(n == NULL){
-      n = notcurses_stdplane(nc);
-    }
     if(scaling == NCSCALE_NONE || scaling == NCSCALE_NONE_HIRES){
       dispcols = lenx;
       disprows = leny;
     }else{
-      ncplane_dim_yx(n, &disprows, &dispcols);
+      if(n == NULL){
+        ncplane_dim_yx(notcurses_stdplane(nc), &disprows, &dispcols);
+      }else{
+        ncplane_dim_yx(n, &disprows, &dispcols);
+      }
       dispcols *= encoding_x_scale(&nc->tcache, bset);
       disprows *= encoding_y_scale(&nc->tcache, bset);
       if(scaling == NCSCALE_SCALE || scaling == NCSCALE_SCALE_HIRES){
@@ -823,7 +843,12 @@ ncplane* ncvisual_render_cells(notcurses* nc, ncvisual* ncv, const struct blitse
     if(flags & NCVISUAL_OPTION_VERALIGNED){
       nopts.flags |= NCPLANE_OPTION_VERALIGNED;
     }
-    if((n = ncplane_create(n, &nopts)) == NULL){
+    if(n){
+      n = ncplane_create(n, &nopts);
+    }else{
+      n = ncpile_create(nc, &nopts);
+    }
+    if(n == NULL){
       return NULL;
     }
     createdn = n;
@@ -868,7 +893,7 @@ ncplane* ncvisual_render_cells(notcurses* nc, ncvisual* ncv, const struct blitse
   bargs.flags = flags;
   bargs.u.cell.placey = placey;
   bargs.u.cell.placex = placex;
-  if(ncvisual_blit(ncv, disprows, dispcols, n, bset, &bargs)){
+  if(ncvisual_blit_internal(ncv, disprows, dispcols, n, bset, &bargs)){
     ncplane_destroy(createdn);
     return NULL;
   }
@@ -877,14 +902,24 @@ ncplane* ncvisual_render_cells(notcurses* nc, ncvisual* ncv, const struct blitse
 
 // create a plane in which to blit the sprixel. |disppixx| and |disppixy| are
 // scaled pixel geometry on output, and unused on input. |placey| and |placex|
-// are used to position the new plane, and reset to 0 on output. |outy| is
-// output height on output, and unused on input.
+// are used to position the new plane, and reset to 0 on output. |outy| and
+// |outx| are true output geometry on output, and unused on input (actual input
+// pixel geometry come from ncv->pixy and ncv->pixx).
+// |pxoffy| and |pxoffx| are pixel offset within the origin cell. they are not
+// included within |disppixx| nor |disppixy|, but count towards |outx| and
+// |outy|. these last two are furthermore clamped to sixel maxima, and |outy|
+// accounts for sixels being a multiple of six pixels tall.
 static ncplane*
 make_sprixel_plane(notcurses* nc, ncplane* parent, ncvisual* ncv,
                    ncscale_e scaling, int* disppixy, int* disppixx,
-                   uint64_t flags, int* outy, int* placey, int* placex){
+                   uint64_t flags, int* outy, int* outx,
+                   int* placey, int* placex, int pxoffy, int pxoffx){
   if(scaling != NCSCALE_NONE && scaling != NCSCALE_NONE_HIRES){
-    ncplane_dim_yx(parent, disppixy, disppixx);
+    if(parent == NULL){
+      ncplane_dim_yx(notcurses_stdplane(nc), disppixy, disppixx);
+    }else{
+      ncplane_dim_yx(parent, disppixy, disppixx);
+    }
     // FIXME why do we clamp only vertical, not horizontal, here?
     if(*placey + *disppixy >= ncplane_dim_y(notcurses_stdplane_const(nc))){
       *disppixy = ncplane_dim_y(notcurses_stdplane_const(nc)) - *placey;
@@ -897,14 +932,19 @@ make_sprixel_plane(notcurses* nc, ncplane* parent, ncvisual* ncv,
     }
     *disppixx *= nc->tcache.cellpixx;
     *disppixy *= nc->tcache.cellpixy;
+    *disppixx += pxoffx;
+    *disppixy += pxoffy;
+    *outx = *disppixx;
     clamp_to_sixelmax(&nc->tcache, disppixy, disppixx, outy, scaling);
     if(scaling == NCSCALE_SCALE || scaling == NCSCALE_SCALE_HIRES){
       scale_visual(ncv, disppixy, disppixx); // can only shrink
+      *outx = *disppixx;
       clamp_to_sixelmax(&nc->tcache, disppixy, disppixx, outy, scaling);
     }
   }else{
-    *disppixx = ncv->pixx;
-    *disppixy = ncv->pixy;
+    *disppixx = ncv->pixx + pxoffx;
+    *disppixy = ncv->pixy + pxoffy;
+    *outx = *disppixx;
     clamp_to_sixelmax(&nc->tcache, disppixy, disppixx, outy, scaling);
   }
   struct ncplane_options nopts = {
@@ -919,11 +959,19 @@ make_sprixel_plane(notcurses* nc, ncplane* parent, ncvisual* ncv,
            | ((flags & NCVISUAL_OPTION_VERALIGNED) ? NCPLANE_OPTION_VERALIGNED : 0),
   };
 //fprintf(stderr, "PLACING NEW PLANE: %d/%d @ %d/%d 0x%016lx\n", nopts.rows, nopts.cols, nopts.y, nopts.x, nopts.flags);
-  // FIXME might need some particular pile
-  ncplane* n = ncplane_create(parent, &nopts);
+  ncplane* n;
+  if(parent == NULL){
+    n = ncpile_create(nc, &nopts);
+  }else{
+    n = ncplane_create(parent, &nopts);
+  }
   if(n == NULL){
     return NULL;
   }
+  // pixel offsets ought be counted for clamping purposes, but not returned
+  // as part of the scaled geometry (they are included in outx/outy).
+  *disppixy -= pxoffy;
+  *disppixx -= pxoffx;
   // we always actually blit to the origin of the plane
   *placey = 0;
   *placex = 0;
@@ -950,20 +998,19 @@ make_sprixel_plane(notcurses* nc, ncplane* parent, ncvisual* ncv,
 ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blitset* bset,
                                 int placey, int placex, int begy, int begx,
                                 int leny, int lenx, ncplane* n, ncscale_e scaling,
-                                uint64_t flags, uint32_t transcolor){
+                                uint64_t flags, uint32_t transcolor,
+                                int pxoffy, int pxoffx){
   ncplane* stdn = notcurses_stdplane(nc);
   if(n == stdn && !(flags & NCVISUAL_OPTION_CHILDPLANE)){
-    logerror("Won't blit bitmaps to the standard plane\n");
+    logerror("won't blit bitmaps to the standard plane\n");
     return NULL;
   }
-  int disppixy = 0, disppixx = 0, outy = 0;
+  int disppixy = 0, disppixx = 0, outy = 0, outx = 0;
   ncplane* createdn = NULL;
   if(n == NULL || (flags & NCVISUAL_OPTION_CHILDPLANE)){ // create plane
-    if(n == NULL){
-      n = notcurses_stdplane(nc);
-    }
     if((createdn = make_sprixel_plane(nc, n, ncv, scaling, &disppixy, &disppixx,
-                                      flags, &outy, &placey, &placex)) == NULL){
+                                      flags, &outy, &outx, &placey, &placex,
+                                      pxoffy, pxoffx)) == NULL){
       return NULL;
     }
     n = createdn;
@@ -971,7 +1018,9 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
     if(scaling != NCSCALE_NONE && scaling != NCSCALE_NONE_HIRES){
       ncplane_dim_yx(n, &disppixy, &disppixx);
       disppixx *= nc->tcache.cellpixx;
+      disppixx += pxoffx;
       disppixy *= nc->tcache.cellpixy;
+      disppixy += pxoffy;
       clamp_to_sixelmax(&nc->tcache, &disppixy, &disppixx, &outy, scaling);
       int absplacex = 0, absplacey = 0;
       if(!(flags & NCVISUAL_OPTION_HORALIGNED)){
@@ -983,8 +1032,8 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
       disppixx -= absplacex * nc->tcache.cellpixx;
       disppixy -= absplacey * nc->tcache.cellpixy;
     }else{
-      disppixx = lenx;
-      disppixy = leny;
+      disppixx = lenx + pxoffx;
+      disppixy = leny + pxoffy;
     }
     if(scaling == NCSCALE_SCALE || scaling == NCSCALE_SCALE_HIRES){
       clamp_to_sixelmax(&nc->tcache, &disppixy, &disppixx, &outy, scaling);
@@ -992,12 +1041,15 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
     }
     clamp_to_sixelmax(&nc->tcache, &disppixy, &disppixx, &outy, scaling);
     // FIXME use a closed form
-    while((outy + nc->tcache.cellpixy - 1) / nc->tcache.cellpixy > ncplane_dim_y(n)){
+    while((outy + nc->tcache.cellpixy - 1) / nc->tcache.cellpixy > (unsigned)ncplane_dim_y(n)){
       outy -= nc->tcache.sprixel_scale_height;
       disppixy = outy;
     }
+    outx = disppixx;
+    disppixx -= pxoffx;
+    disppixy -= pxoffy;
   }
-//fprintf(stderr, "pblit: %dx%d <- %dx%d of %d/%d stride %u @%dx%d %p %u\n", disppixy, disppixx, begy, begx, ncv->pixy, ncv->pixx, ncv->rowstride, placey, placex, ncv->data, nc->tcache.cellpixx);
+  logdebug("pblit: %dx%d <- %dx%d of %d/%d stride %u @%dx%d %p %u\n", disppixy, disppixx, begy, begx, ncv->pixy, ncv->pixx, ncv->rowstride, placey, placex, ncv->data, nc->tcache.cellpixx);
   blitterargs bargs;
   if(flags & NCVISUAL_OPTION_ADDALPHA){
     bargs.transcolor = transcolor | 0x1000000ul;
@@ -1010,22 +1062,38 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
   bargs.lenx = lenx;
   bargs.flags = flags;
   bargs.u.pixel.colorregs = nc->tcache.color_registers;
+  bargs.u.pixel.pxoffy = pxoffy;
+  bargs.u.pixel.pxoffx = pxoffx;
+  int cols = outx / nc->tcache.cellpixx + !!(outx % nc->tcache.cellpixx);
+  int rows = outy / nc->tcache.cellpixy + !!(outy % nc->tcache.cellpixy);
+  logdebug("cblit: rows/cols: %dx%d plane: %d/%d out: %d/%d\n", rows, cols, ncplane_dim_y(n), ncplane_dim_x(n), outy, outx);
   if(n->sprite == NULL){
-    int cols = disppixx / nc->tcache.cellpixx + !!(disppixx % nc->tcache.cellpixx);
-    int rows = outy / nc->tcache.cellpixy + !!(outy % nc->tcache.cellpixy);
     if((n->sprite = sprixel_alloc(&nc->tcache, n, rows, cols)) == NULL){
       ncplane_destroy(createdn);
       return NULL;
     }
+    if((n->tam = create_tam(rows, cols)) == NULL){
+      ncplane_destroy(createdn);
+      return NULL;;
+    }
   }else{
     n->sprite = sprixel_recycle(n);
+    if(n->sprite->dimy != rows || n->sprite->dimx != cols){
+      destroy_tam(n);
+      if((n->tam = create_tam(rows, cols)) == NULL){
+        ncplane_destroy(createdn);
+        return NULL;
+      }
+    }
+    n->sprite->dimx = cols;
+    n->sprite->dimy = rows;
   }
   bargs.u.pixel.spx = n->sprite;
   // FIXME need to pull off the ncpile's sprixellist if anything below fails!
   // at this point, disppixy/disppixx are the output geometry (might be larger
   // than scaledy/scaledx for sixel), while scaledy/scaledx are the scaled
   // geometry. cells occupied are determined based off disppixy/disppixx.
-  if(ncvisual_blit(ncv, disppixy, disppixx, n, bset, &bargs)){
+  if(ncvisual_blit_internal(ncv, disppixy, disppixx, n, bset, &bargs)){
     ncplane_destroy(createdn);
     return NULL;
   }
@@ -1059,6 +1127,12 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
 //fprintf(stderr, "ABOUT TO RESIZE: yoff/xoff: %d/%d\n",  placey, placex);
   // FIXME might need shrink down the TAM and kill unnecessary auxvecs
   if(ncplane_resize(n, 0, 0, s->dimy, s->dimx, placey, placex, s->dimy, s->dimx)){
+    // if we blow up here, then we've got a TAM sized to the sprixel, rather
+    // than the plane. running it through destroy_tam() via ncplane_destroy()
+    // will use incorrect bounds for scrubbing said TAM. do it manually here.
+    cleanup_tam(n->tam, rows, cols);
+    free(n->tam);
+    n->tam = NULL;
     sprixel_hide(bargs.u.pixel.spx);
     ncplane_destroy(createdn);
     return NULL;
@@ -1068,7 +1142,7 @@ ncplane* ncvisual_render_pixels(notcurses* nc, ncvisual* ncv, const struct blits
   return n;
 }
 
-ncplane* ncvisual_render(notcurses* nc, ncvisual* ncv, const struct ncvisual_options* vopts){
+ncplane* ncvisual_blit(notcurses* nc, ncvisual* ncv, const struct ncvisual_options* vopts){
   const struct blitset* bset;
   int leny, lenx;
   if(ncvisual_blitset_geom(nc, &nc->tcache, ncv, vopts, NULL, NULL, NULL, NULL,
@@ -1095,11 +1169,30 @@ ncplane* ncvisual_render(notcurses* nc, ncvisual* ncv, const struct ncvisual_opt
   }else{
     n = ncvisual_render_pixels(nc, ncv, bset, placey, placex, begy, begx,
                                leny, lenx, n, scaling,
-                               vopts ? vopts->flags : 0, transcolor);
+                               vopts ? vopts->flags : 0, transcolor,
+                               vopts ? vopts->pxoffy : 0,
+                               vopts ? vopts->pxoffx : 0);
   }
   return n;
 }
 
+// compatability wrapper around ncvisual_blit that provides the standard plane
+// plus NCVISUAL_OPTION_CHILDPLANE if vopts->n is NULL.
+ncplane* ncvisual_render(notcurses* nc, ncvisual* ncv, const struct ncvisual_options* vopts){
+  struct ncvisual_options fakevopts;
+  if(vopts == NULL){
+    memset(&fakevopts, 0, sizeof(fakevopts));
+  }else{
+    memcpy(&fakevopts, vopts, sizeof(fakevopts));
+  }
+  vopts = &fakevopts;
+  if(vopts->n == NULL){
+    fakevopts.n = notcurses_stdplane(nc);
+    fakevopts.flags |= NCVISUAL_OPTION_CHILDPLANE;
+  }
+  return ncvisual_blit(nc, ncv, vopts);
+}
+
 ncvisual* ncvisual_from_plane(const ncplane* n, ncblitter_e blit, int begy, int begx,
                               int leny, int lenx){
   int py, px;
