@@ -5,7 +5,7 @@
 
 static void
 printbanner(fbuf* f){
-  fbuf_puts(f, "built without multimedia support\n");
+  fbuf_puts(f, "built without multimedia support" NL);
 }
 
 const ncvisual_implementation local_visual_implementation = {
