@@ -9,7 +9,6 @@
 #include <unistd.h>
 #include <iostream>
 #include <inttypes.h>
-#include <ncpp/Input.hh>
 #include <ncpp/Direct.hh>
 #include <ncpp/Visual.hh>
 #include <ncpp/NotCurses.hh>
@@ -48,7 +47,7 @@ struct marshal {
 auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
               const struct timespec* abstime, void* vmarshal) -> int {
   struct marshal* marsh = static_cast<struct marshal*>(vmarshal);
-  NotCurses &nc = NotCurses::get_instance ();
+  NotCurses &nc = NotCurses::get_instance();
   auto start = static_cast<struct timespec*>(ncplane_userptr(vopts->n));
   if(!start){
     start = static_cast<struct timespec*>(malloc(sizeof(struct timespec)));
@@ -104,26 +103,27 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     }else{
       keyp = nc.get(false, &ni);
     }
+    if(keyp == 0){
+      break;
+    }
     // we don't care about key release events, especially the enter
     // release that starts so many interactive programs under Kitty
     if(ni.evtype == EvType::Release){
       continue;
     }
-    if(keyp == 0){
-      break;
-    }
     if(keyp == ' '){
-      if((keyp = nc.get(true)) == (uint32_t)-1){
-        ncplane_destroy(subp);
-        return -1;
-      }
+      do{
+        if((keyp = nc.get(true, &ni)) == (uint32_t)-1){
+          ncplane_destroy(subp);
+          return -1;
+        }
+      }while(ni.id != 'q' && (ni.evtype == EvType::Release || ni.id != ' '));
     }
     // if we just hit a non-space character to unpause, ignore it
-    if(keyp == ' '){ // space for unpause
-      continue;
-    }
     if(keyp == NCKey::Resize){
       return 0;
+    }else if(keyp == ' '){ // space for unpause
+      continue;
     }else if(keyp == 'L' && ni.ctrl && !ni.alt){
       nc.refresh(nullptr, nullptr);
       continue;
@@ -145,6 +145,8 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     }else if(keyp == NCKey::Left){
       // FIXME move backwards
       continue;
+    }else if(keyp != 'q'){
+      continue;
     }
     ncplane_destroy(subp);
     return 1;
@@ -405,31 +407,40 @@ int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
         vopts.blitter = marsh.blitter;
         if(!loop){
           if(displaytime < 0){
-            stdn->printf(0, NCAlign::Center, "press a key to advance");
+            stdn->printf(0, NCAlign::Center, "press space to advance");
             if(!nc.render()){
               ncplane_destroy(n);
               return -1;
             }
-            uint32_t ie = nc.get(true);
-            if(ie == (uint32_t)-1){
-              return -1;
-            }else if(ie == 'q'){
-              return 0;
-            }else if(ie == 'L'){
-              --i;
-              nc.refresh(nullptr, nullptr);
-            }else if(ie >= '0' && ie <= '6'){
-              blitter = vopts.blitter = static_cast<ncblitter_e>(ie - '0');
-              --i; // rerun same input with the new blitter
-            }else if(ie >= '7' && ie <= '9'){
-              --i; // just absorb the input
-            }else if(ie == NCKey::Resize){
-              --i; // rerun with the new size
-              if(!nc.refresh(&dimy, &dimx)){
-                ncplane_destroy(n);
+            ncinput ni;
+            do{
+              do{
+                nc.get(true, &ni);
+              }while(ni.evtype == EvType::Release);
+              if(ni.id == (uint32_t)-1){
                 return -1;
+              }else if(ni.id == 'q'){
+                return 0;
+              }else if(ni.id == 'L'){
+                --i;
+                nc.refresh(nullptr, nullptr);
+                break;
+              }else if(ni.id >= '0' && ni.id <= '6'){
+                blitter = vopts.blitter = static_cast<ncblitter_e>(ni.id - '0');
+                --i; // rerun same input with the new blitter
+                break;
+              }else if(ni.id >= '7' && ni.id <= '9'){
+                --i; // just absorb the input
+                break;
+              }else if(ni.id == NCKey::Resize){
+                --i; // rerun with the new size
+                if(!nc.refresh(&dimy, &dimx)){
+                  ncplane_destroy(n);
+                  return -1;
+                }
+                break;
               }
-            }
+            }while(ni.id != ' ');
           }else{
             // FIXME do we still want to honor keybindings when timing out?
             struct timespec ts;
