@@ -27,18 +27,12 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y, htole(0xe61c28ff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
     CHECK(0 == ncvisual_resize(ncv, 1, 1));
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -54,20 +48,14 @@ TEST_CASE("Bitmaps") {
     REQUIRE(nullptr != ncv);
     auto nn = ncplane_dup(n_, nullptr);
     REQUIRE(nullptr != nn);
-    struct ncvisual_options vopts = {
-      .n = nn,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
+    struct ncvisual_options vopts{};
+    vopts.n = nn;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE;
     int maxy, maxx;
     ncplane_pixelgeom(n_, nullptr, nullptr, nullptr, nullptr, &maxy, &maxx);
     CHECK(0 == ncvisual_resize(ncv, maxy, maxx));
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nn == n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -90,18 +78,11 @@ TEST_CASE("Bitmaps") {
       .flags = 0, .margin_b = 0, .margin_r = 0,
     };
     auto n = ncplane_create(n_, &nopts);
-    struct ncvisual_options vopts = {
-      .n = n,
-      .scaling = NCSCALE_NONE,
-      .y = 0,
-      .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    CHECK(nullptr == ncvisual_render(nc_, ncv, &vopts));
+    struct ncvisual_options vopts{};
+    vopts.n = n;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE;
+    CHECK(nullptr == ncvisual_blit(nc_, ncv, &vopts));
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
     CHECK(0 == ncplane_destroy(n));
@@ -120,18 +101,11 @@ TEST_CASE("Bitmaps") {
       .flags = 0, .margin_b = 0, .margin_r = 0,
     };
     auto n = ncplane_create(n_, &nopts);
-    struct ncvisual_options vopts = {
-      .n = n,
-      .scaling = NCSCALE_NONE,
-      .y = 0,
-      .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    CHECK(nullptr == ncvisual_render(nc_, ncv, &vopts));
+    struct ncvisual_options vopts{};
+    vopts.n = n;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE;
+    CHECK(nullptr == ncvisual_blit(nc_, ncv, &vopts));
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
     CHECK(0 == ncplane_destroy(n));
@@ -152,18 +126,11 @@ TEST_CASE("Bitmaps") {
       .flags = 0, .margin_b = 0, .margin_r = 0,
     };
     auto n = ncplane_create(n_, &nopts);
-    struct ncvisual_options vopts = {
-      .n = n,
-      .scaling = NCSCALE_NONE,
-      .y = 0,
-      .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    CHECK(nullptr != ncvisual_render(nc_, ncv, &vopts));
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    CHECK(nullptr != ncvisual_blit(nc_, ncv, &vopts));
     CHECK(0 > ncplane_putchar_yx(n, ' ', 0, 0));
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
@@ -177,17 +144,11 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y, htole(0xe61c28ff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr, .scaling = NCSCALE_NONE,
-      .y = 0,
-      .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto botn = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto botn = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != botn);
     // should just have a red plane
     CHECK(0 == notcurses_render(nc_));
@@ -196,7 +157,7 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v2(x * y, htole(0x8142f1ff));
     auto ncv2 = ncvisual_from_rgba(v2.data(), y, sizeof(decltype(v2)::value_type) * x, x);
     REQUIRE(nullptr != ncv2);
-    auto topn = ncvisual_render(nc_, ncv2, &vopts);
+    auto topn = ncvisual_blit(nc_, ncv2, &vopts);
     REQUIRE(nullptr != topn);
     // should have a yellow plane partially obscuring a red one
     CHECK(0 == notcurses_render(nc_));
@@ -221,9 +182,10 @@ TEST_CASE("Bitmaps") {
     auto ncv = ncvisual_from_file(find_data("worldmap.png").get());
     REQUIRE(ncv);
     struct ncvisual_options vopts{};
+    vopts.n = n_;
     vopts.blitter = NCBLIT_PIXEL;
-    vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-    auto newn = ncvisual_render(nc_, ncv, &vopts);
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto newn = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(newn);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(newn));
@@ -240,17 +202,11 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y, htole(0xffffffff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -269,18 +225,20 @@ TEST_CASE("Bitmaps") {
     auto bigp = ncplane_create(n_, &nopts);
     REQUIRE(bigp);
     vopts.n = bigp;
+    vopts.flags &= ~NCVISUAL_OPTION_CHILDPLANE;
     uint64_t white = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, 0xff, 0xff, 0xff);
     ncplane_set_base(bigp, "x", 0, white);
-    CHECK(vopts.n == ncvisual_render(nc_, ncv, &vopts));
+    CHECK(vopts.n == ncvisual_blit(nc_, ncv, &vopts));
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 4, ncv->pixx * 4));
     CHECK(4 * nc_->tcache.cellpixy == ncv->pixy);
     CHECK(4 * nc_->tcache.cellpixx == ncv->pixx);
     vopts.y = 1;
     vopts.x = 6;
-    vopts.n = nullptr;
+    vopts.n = n_;
     vopts.scaling = NCSCALE_NONE;
-    auto infn = ncvisual_render(nc_, ncv, &vopts);
+    vopts.flags |= NCVISUAL_OPTION_CHILDPLANE;
+    auto infn = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(infn);
     if(nc_->tcache.sprixel_scale_height == 6){
       if(4 * nc_->tcache.cellpixy % 6){
@@ -298,7 +256,7 @@ TEST_CASE("Bitmaps") {
     CHECK(ncv->pixy == 8);
     CHECK(ncv->pixx == 8);
     vopts.x = 11;
-    auto resizen = ncvisual_render(nc_, ncv, &vopts);
+    auto resizen = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(resizen);
     CHECK((8 + nc_->tcache.cellpixy - 1) / nc_->tcache.cellpixy == ncplane_dim_y(resizen));
     CHECK((8 + nc_->tcache.cellpixx - 1) / nc_->tcache.cellpixx == ncplane_dim_x(resizen));
@@ -312,7 +270,7 @@ TEST_CASE("Bitmaps") {
   }
 
   // create an image of exactly 1 cell, inflate it, scale it, and compare the
-  // resulting geometries for equality
+  // resulting geometries for (rough) equality
   SUBCASE("InflateVsScale") {
     // first, assemble a visual equivalent to 1 cell
     auto y = nc_->tcache.cellpixy;
@@ -320,17 +278,11 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y, htole(0xff7799dd));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     struct ncplane_options nopts = {
       .y = 2,
@@ -340,26 +292,39 @@ TEST_CASE("Bitmaps") {
       .userptr = nullptr, .name = "scale", .resizecb = nullptr,
       .flags = 0, .margin_b = 0, .margin_r = 0,
     };
-    auto nres = ncplane_create(n_, &nopts);
-    REQUIRE(nullptr != nres);
-    vopts.n = nres;
+    auto scal = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != scal);
+    vopts.n = scal;
     vopts.scaling = NCSCALE_SCALE;
-    ncvisual_render(nc_, ncv, &vopts);
+    auto bmap = ncvisual_blit(nc_, ncv, &vopts);
+    REQUIRE(nullptr != bmap);
     CHECK(4 == ncplane_dim_x(vopts.n));
-    CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 4, ncv->pixx * 4));
-    vopts.n = nullptr;
+    CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 5, ncv->pixx * 4));
+    vopts.n = scal;
     vopts.y = 2;
     vopts.x = 5;
     vopts.scaling = NCSCALE_NONE;
-    auto ninf = ncvisual_render(nc_, ncv, &vopts);
+    auto ninf = ncvisual_blit(nc_, ncv, &vopts);
+    ncplane_set_name(ninf, "ninf");
+notcurses_debug(nc_, stderr);
     REQUIRE(nullptr != ninf);
-    CHECK(ncplane_dim_y(nres) == ncplane_dim_y(ninf));
-    CHECK(ncplane_dim_x(nres) == ncplane_dim_x(ninf));
+    // y of scaled might not be exactly equal to y of inflated since one
+    // is based around what can be fit into a specific space, whereas the
+    // other is based on a multiple of the original image size, which might
+    // not be a multiple of the sixel quantum.
+    CHECK(5 == ncplane_dim_y(scal));
+    if(5 >= ncplane_dim_y(ninf)){
+      CHECK(5 == ncplane_dim_y(ninf));
+    }else{
+      CHECK(6 == ncplane_dim_y(ninf));
+    }
+    CHECK(ncplane_dim_x(scal) == ncplane_dim_x(ninf));
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
     CHECK(0 == ncplane_destroy(n));
     CHECK(0 == ncplane_destroy(ninf));
-    CHECK(0 == ncplane_destroy(nres));
+    CHECK(0 == ncplane_destroy(scal));
+    CHECK(0 == ncplane_destroy(bmap));
   }
 
   // test NCVISUAL_OPTIONS_CHILDPLANE + stretch + (null) alignment
@@ -388,13 +353,14 @@ TEST_CASE("Bitmaps") {
                NCVISUAL_OPTION_HORALIGNED |
                NCVISUAL_OPTION_VERALIGNED,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     auto y = nc_->tcache.cellpixy * 6;
     auto x = nc_->tcache.cellpixx;
     std::vector<uint32_t> v(x * y, htole(0xffffffff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(18 == ncplane_dim_y(child));
     CHECK(5 == ncplane_dim_x(child));
@@ -430,13 +396,14 @@ TEST_CASE("Bitmaps") {
                NCVISUAL_OPTION_HORALIGNED |
                NCVISUAL_OPTION_VERALIGNED,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     auto y = nc_->tcache.cellpixy * 6;
     auto x = nc_->tcache.cellpixx;
     std::vector<uint32_t> v(x * y, htole(0xffffffff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(6 == ncplane_dim_y(child));
     CHECK(1 == ncplane_dim_x(child));
@@ -454,17 +421,11 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y, htole(0xffffffff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = y, .lenx = x,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -492,17 +453,11 @@ TEST_CASE("Bitmaps") {
     }
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = y, .lenx = x,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -530,17 +485,11 @@ TEST_CASE("Bitmaps") {
     }
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = y, .lenx = x,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     auto s = n->sprite;
     REQUIRE(nullptr != s);
@@ -576,17 +525,11 @@ TEST_CASE("Bitmaps") {
     }
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 2, .x = 2,
-      .begy = 0, .begx = 0,
-      .leny = y, .lenx = x,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     ncvisual_destroy(ncv);
     CHECK(0 == notcurses_render(nc_));
@@ -643,10 +586,11 @@ TEST_CASE("Bitmaps") {
     auto ncv = ncvisual_from_file(find_data("worldmap.png").get());
     REQUIRE(ncv);
     struct ncvisual_options vopts{};
+    vopts.n = n_;
     vopts.blitter = NCBLIT_PIXEL;
-    vopts.flags = NCVISUAL_OPTION_NODEGRADE;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
     vopts.scaling = NCSCALE_STRETCH;
-    auto newn = ncvisual_render(nc_, ncv, &vopts);
+    auto newn = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(newn);
     ncplane_move_bottom(newn);
     CHECK(0 == notcurses_render(nc_));
@@ -673,22 +617,18 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y * 4, htole(0xffccccff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
+    REQUIRE(nullptr != n);
     for(int i = 0 ; i <= ncplane_dim_x(n_) ; ++i){
       CHECK(0 == ncplane_move_yx(n, 0, i));
       CHECK(0 == notcurses_render(nc_));
     }
-    REQUIRE(nullptr != n);
+    ncvisual_destroy(ncv);
+    CHECK(0 == ncplane_destroy(n));
   }
 
   SUBCASE("BitmapMoveOffscreenRight") {
@@ -698,22 +638,19 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y * 4, htole(0xffccccff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = ncplane_dim_x(n_) - 3,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    vopts.x = ncplane_dim_x(n_) - 3;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
+    REQUIRE(nullptr != n);
     for(int i = ncplane_dim_x(n_) - 3 ; i >= 0 ; --i){
       CHECK(0 == ncplane_move_yx(n, 0, i));
       CHECK(0 == notcurses_render(nc_));
     }
-    REQUIRE(nullptr != n);
+    ncvisual_destroy(ncv);
+    CHECK(0 == ncplane_destroy(n));
   }
 
   SUBCASE("BitmapMoveOffscreenDown") {
@@ -723,22 +660,18 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y * 4, htole(0xffccccff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
+    REQUIRE(nullptr != n);
     for(int i = 0 ; i <= ncplane_dim_y(n_) ; ++i){
       CHECK(0 == ncplane_move_yx(n, i, 0));
       CHECK(0 == notcurses_render(nc_));
     }
-    REQUIRE(nullptr != n);
+    ncvisual_destroy(ncv);
+    CHECK(0 == ncplane_destroy(n));
   }
 
   SUBCASE("BitmapMoveOffscreenUp") {
@@ -748,22 +681,52 @@ TEST_CASE("Bitmaps") {
     std::vector<uint32_t> v(x * y * 4, htole(0xffccccff));
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_NONE,
-      .y = ncplane_dim_y(n_) - 3, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_PIXEL,
-      .flags = NCVISUAL_OPTION_NODEGRADE,
-      .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.y = ncplane_dim_y(n_) - 3;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
+    REQUIRE(nullptr != n);
     for(int i = ncplane_dim_y(n_) - 3 ; i >= 0 ; --i){
       CHECK(0 == ncplane_move_yx(n, i, 0));
       CHECK(0 == notcurses_render(nc_));
     }
+    ncvisual_destroy(ncv);
+    CHECK(0 == ncplane_destroy(n));
+  }
+
+  // smoothly move a bitmap diagonally across the screen
+  SUBCASE("BitmapSmoothMove") {
+    // first, assemble a visual equivalent to 2x2 cells
+    auto y = nc_->tcache.cellpixy * 2;
+    auto x = nc_->tcache.cellpixx * 2;
+    std::vector<uint32_t> v(x * y * 4, htole(0xffccccff));
+    auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
+    REQUIRE(nullptr != ncv);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.blitter = NCBLIT_PIXEL;
+    vopts.flags = NCVISUAL_OPTION_NODEGRADE | NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
+    auto xpx = (ncplane_dim_x(n_) - 2) * nc_->tcache.cellpixx;
+    auto ypx = (ncplane_dim_y(n_) - 2) * nc_->tcache.cellpixy;
+    double xyrat = (double)ypx / xpx;
+    for(unsigned xat = 0 ; xat < xpx ; ++xat){
+      vopts.x = xat / nc_->tcache.cellpixx;
+      vopts.pxoffx = xat % nc_->tcache.cellpixx;
+      int yat = xat * xyrat;
+      vopts.y = yat / nc_->tcache.cellpixy;
+      vopts.pxoffy = yat % nc_->tcache.cellpixy;
+      CHECK(0 == ncplane_destroy(n));
+      n = ncvisual_blit(nc_, ncv, &vopts);
+      REQUIRE(nullptr != n);
+      CHECK(0 == notcurses_render(nc_));
+    }
+    ncvisual_destroy(ncv);
+    CHECK(0 == ncplane_destroy(n));
+    CHECK(0 == notcurses_render(nc_));
   }
 
   CHECK(!notcurses_stop(nc_));
