@@ -74,18 +74,20 @@ TEST_CASE("Visual") {
     struct ncvisual_options vopts{};
     vopts.x = NCALIGN_CENTER;
     vopts.y = NCALIGN_CENTER;
-    vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED;
-    auto p = ncvisual_render(nc_, ncv, &vopts);
+    vopts.n = n_;
+    vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED
+                    | NCVISUAL_OPTION_CHILDPLANE;
+    auto p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
     CHECK(0 == ncvisual_resize(ncv, 20, 20));
-    p = ncvisual_render(nc_, ncv, &vopts);
+    p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
     CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-    p = ncvisual_render(nc_, ncv, &vopts);
+    p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
@@ -96,7 +98,7 @@ TEST_CASE("Visual") {
   SUBCASE("VisualAligned") {
     const uint32_t pixels[4] = { htole(0xffff0000), htole(0xff00ff00), htole(0xff0000ff), htole(0xffffffff) };
     ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0,
       .x = NCALIGN_LEFT,
@@ -105,12 +107,13 @@ TEST_CASE("Visual") {
       .leny = 2,
       .lenx = 2,
       .blitter = NCBLIT_1x1,
-      .flags = NCVISUAL_OPTION_HORALIGNED,
+      .flags = NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_CHILDPLANE,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     auto ncv = ncvisual_from_rgba(pixels, 2, 2 * sizeof(*pixels), 2);
     REQUIRE(nullptr != ncv);
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
@@ -125,15 +128,17 @@ TEST_CASE("Visual") {
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
     struct ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0, .x = 0,
       .begy = 0, .begx = 0,
       .leny = 5, .lenx = 8,
       .blitter = NCBLIT_1x1,
-      .flags = 0, .transcolor = 0,
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
+      .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(5 == ncplane_dim_y(n));
     CHECK(8 == ncplane_dim_x(n));
@@ -150,15 +155,17 @@ TEST_CASE("Visual") {
     auto ncv = ncvisual_from_rgba(v.data(), 1, sizeof(decltype(v)::value_type), 1);
     REQUIRE(nullptr != ncv);
     struct ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_STRETCH,
       .y = 0, .x = 0,
       .begy = 0, .begx = 0,
       .leny = 0, .lenx = 0,
       .blitter = NCBLIT_1x1,
-      .flags = 0, .transcolor = 0,
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
+      .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(0 == notcurses_render(nc_));
     REQUIRE(nullptr != n);
     CHECK(dimy == ncplane_dim_y(n));
@@ -193,8 +200,9 @@ TEST_CASE("Visual") {
       .leny = 5, .lenx = 8,
       .blitter = NCBLIT_1x1,
       .flags = 0, .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(5 == ncplane_dim_y(n));
     CHECK(8 == ncplane_dim_x(n));
@@ -216,8 +224,9 @@ TEST_CASE("Visual") {
       .leny = 0, .lenx = 0,
       .blitter = NCBLIT_1x1,
       .flags = 0, .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto newn = ncvisual_render(nc_, ncv, &vopts);
+    auto newn = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 3, ncv->pixx * 3));
     CHECK(6 == ncv->pixy);
@@ -239,7 +248,7 @@ TEST_CASE("Visual") {
       }
     }
     REQUIRE(newn);
-    auto enewn = ncvisual_render(nc_, ncv, &vopts);
+    auto enewn = ncvisual_blit(nc_, ncv, &vopts);
     int newy, newx;
     ncplane_dim_yx(enewn, &newy, &newx);
     CHECK(6 == newy);
@@ -260,7 +269,7 @@ TEST_CASE("Visual") {
     struct ncvisual_options opts{};
     opts.blitter = NCBLIT_1x1;
     opts.n = ncp_;
-    CHECK(ncp_ == ncvisual_render(nc_, ncv, &opts));
+    CHECK(ncp_ == ncvisual_blit(nc_, ncv, &opts));
     CHECK(0 == notcurses_render(nc_));
     for(int y = 0 ; y < dimy ; ++y){
       for(int x = 0 ; x < dimx ; ++x){
@@ -288,7 +297,7 @@ TEST_CASE("Visual") {
     struct ncvisual_options opts{};
     opts.blitter = NCBLIT_1x1;
     opts.n = ncp_;
-    CHECK(nullptr != ncvisual_render(nc_, ncv, &opts));
+    CHECK(nullptr != ncvisual_blit(nc_, ncv, &opts));
     CHECK(0 == notcurses_render(nc_));
     for(int y = 0 ; y < dimy ; ++y){
       for(int x = 0 ; x < dimx ; ++x){
@@ -328,7 +337,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x1;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX ; ++x){
@@ -368,7 +377,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x2;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX / 2 ; ++x){
@@ -426,7 +435,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x2;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX / 2 ; ++x){
@@ -466,7 +475,7 @@ TEST_CASE("Visual") {
       auto ncv = ncvisual_from_rgba(pixels, 2, 2 * sizeof(*pixels), 2);
       REQUIRE(nullptr != ncv);
       struct ncvisual_options vopts = {
-        .n = nullptr,
+        .n = n_,
         .scaling = NCSCALE_NONE,
         .y = 0,
         .x = 0,
@@ -475,10 +484,11 @@ TEST_CASE("Visual") {
         .leny = 0,
         .lenx = 0,
         .blitter = NCBLIT_2x2,
-        .flags = 0,
+        .flags = NCVISUAL_OPTION_CHILDPLANE,
         .transcolor = 0,
+        .pxoffy = 0, .pxoffx = 0,
       };
-      auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+      auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
       REQUIRE(nullptr != ncvp);
       int dimy, dimx;
       ncplane_dim_yx(ncvp, &dimy, &dimx);
@@ -510,7 +520,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -519,10 +529,11 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
         int dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
@@ -557,7 +568,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -566,10 +577,11 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
         int dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
@@ -609,7 +621,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -618,10 +630,11 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
         int dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
@@ -660,7 +673,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -669,10 +682,11 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
         int dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
@@ -692,7 +706,7 @@ TEST_CASE("Visual") {
     }
   }
 
-  // test NCVISUAL_OPTIONS_CHILDPLANE + stretch + null alignment
+  // test NCVISUAL_OPTION_CHILDPLANE + stretch + null alignment
   SUBCASE("ImageChildScaling") {
     struct ncplane_options opts = {
       .y = 0, .x = 0,
@@ -707,15 +721,16 @@ TEST_CASE("Visual") {
     auto parent = ncplane_create(n_, &opts);
     REQUIRE(parent);
     struct ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0,
       .x = 0,
       .begy = 0, .begx = 0,
       .leny = 0, .lenx = 0,
       .blitter = NCBLIT_1x1,
-      .flags = 0,
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     const uint32_t pixels[16] = {
       htole(0xffffffff), htole(0xffffffff), htole(0xffc0ffff), htole(0xffffc0ff),
@@ -725,7 +740,7 @@ TEST_CASE("Visual") {
     };
     auto ncv = ncvisual_from_rgba(pixels, 4, 16, 4);
     REQUIRE(ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(4 == ncplane_dim_y(child));
     CHECK(4 == ncplane_dim_x(child));
@@ -736,7 +751,7 @@ TEST_CASE("Visual") {
     vopts.n = parent,
     vopts.scaling = NCSCALE_STRETCH,
     vopts.flags = NCVISUAL_OPTION_CHILDPLANE;
-    child = ncvisual_render(nc_, ncv, &vopts);
+    child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(20 == ncplane_dim_y(child));
     CHECK(20 == ncplane_dim_x(child));
@@ -771,11 +786,12 @@ TEST_CASE("Visual") {
                NCVISUAL_OPTION_HORALIGNED |
                NCVISUAL_OPTION_VERALIGNED,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     const uint32_t pixels[1] = { htole(0xffffffff) };
     auto ncv = ncvisual_from_rgba(pixels, 1, 4, 1);
     REQUIRE(ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(1 == ncplane_dim_y(child));
     CHECK(1 == ncplane_dim_x(child));
